@@ -61,7 +61,7 @@
 ;; 92.9.8 by K.Handa
 ;;;###autoload
 (defvar keyboard-allow-latin-input nil
-  "If non-nil, \"ESC , Fe\" and \"ESC - Fe\" are used for inputting
+  "If non-nil, "ESC , Fe" and "ESC - Fe" are used for inputting
 Latin characters.")
 
 ;; common global variables of internal use
