@@ -614,14 +614,14 @@
   "CCL program to read VISCII 1.1")
 
 (define-ccl-program ccl-write-viscii
-  (` (1
+  '(1
      ((read r0)
       (loop
        (if (r0 < 128)
 	   (write-read-repeat r0)
-	 (if (r0 != (, lc-prv11))
+	 (if (r0 != 154)
 	     (write-read-repeat r0)
-	   ((read-if (r0 == (, lc-vn-1))
+	   ((read-if (r0 == 163)
 	     ((read r0)
 	      (r0 -= 160)
 	      (write-read-repeat
@@ -633,7 +633,7 @@
 	      224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239
 	      240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 0
 	      ]))
-	     (if (r0 == (, lc-vn-2))
+	     (if (r0 == 164)
 		 ((read r0)
 		  (r0 -= 160)
 		  (write-read-repeat
@@ -645,7 +645,7 @@
 	        192 193 194 195 196 197 255   6 200 201 202 203 204 205 206 155
 	        208 185 210 211 212 160 153 154 158 217 218 157 156 221 148   0
 		]))
-	       (write-read-repeat r0))))))))))
+	       (write-read-repeat r0)))))))))
   "CCL program to write VISCII 1.1")
 
 (make-coding-system
@@ -685,14 +685,14 @@
   "CCL program to read VSCII-1.")
 
 (define-ccl-program ccl-write-vscii
-  (` (1
+  '(1
      ((read r0)
       (loop
 	  (if (r0 < 128)
 	      (write-read-repeat r0)
-	    (if (r0 != (, lc-prv11))
+	    (if (r0 != 154)
 		(write-read-repeat r0)
-	      (read-if (r0 == (, lc-vn-1))
+	      (read-if (r0 == 163)
 	       ((read r0)
 		(r0 -= 160)
 		(write-read-repeat
@@ -704,7 +704,7 @@
 	      181 184 169 183 182 168 247 201 204 208 170 206 215 221 220 216
 	      174 249 223 227 171 226 225 228 244 239 243 242 241 253 238   0
 	      ]))
-	       (if (r0 == (, lc-vn-2))
+	       (if (r0 == 164)
 		((read r0)
 		 (r0 -= 160)
 		 (write-read-repeat
@@ -716,7 +716,7 @@
 	        128 131 162 130 129 161   6 195 135 138 163 136 141 144 143 142
 	        167   0 146 149 164 148 147 150   2 157   1 159 158  22 156   0
 		]))
-		(write-read-repeat r0)))))))))
+		(write-read-repeat r0))))))))
   "CCL program to write VSCII-1.")
 
 (make-coding-system
