@@ -913,7 +913,33 @@ This function runs `rmail-get-new-mail-hook' before saving the updated file."
       (if (file-exists-p tofile)
 	  (let (size)
 	    (goto-char (point-max))
-	    (setq size (nth 1 (insert-file-contents tofile)))
+	    ;; At first, read the file without converting coding-system.
+	    (setq size (nth 1 (let (file-coding-system-for-read)
+				(insert-file-contents tofile))))
+	    ;; Then, convert the contents if necessary.
+	    (if (> size 0)
+		(cond
+		 ((looking-at "^From ")
+		  ;; New mails.  Since the contents may be a mixture
+		  ;; of various coding-systems, we must decode one
+		  ;; mail by one.
+		  (while (null (eobp))
+		    (let ((from (point)))
+		      (re-search-forward "^From " nil 'mv)
+		      (rmail-decode-coding-system from (point)))))
+		 ((looking-at "BABYL OPTIONS:\\|\^L")
+		  ;; Babyl format, i.e. not a new mail.  We had better
+		  ;; not to convert large region at once.
+		  (while (null (eobp))
+		    (let ((max-size (* 1024 1024))
+			  (from (point)))
+		      (goto-char (+ from max-size))
+		      (re-search-forward "\^_" nil 'mv)
+		      (rmail-decode-coding-system from (point)))))
+		 (t
+		  ;; Perhaps, MMDF format.  Since I don't know how to
+		  ;; deal with it, convert all data at once.
+		  (rmail-decode-coding-system (point) (point-max)))))
 	    (goto-char (point-max))
 	    (or (= (preceding-char) ?\n)
 		(zerop size)
@@ -923,6 +949,31 @@ This function runs `rmail-get-new-mail-hook' before saving the updated file."
       (setq files (cdr files)))
     delete-files))
 
+(defvar mail-coding-system '*junet*)
+
+(defun rmail-decode-coding-system (from to)
+  (let (coding-system)
+    ;; At first, detect the coding-system of the region and set it to
+    ;; `coding-sytem'.
+    (let ((detected-coding-system (code-detect-region from to))
+	  (coding (get-code mail-coding-system)))
+      (if (listp detected-coding-system)
+	  ;; Something other than ASCII was found.  If a coding-system
+	  ;; of which information is same as `mail-coding-system' is
+	  ;; in the list of detected coding-systems, use it, else use
+	  ;; the coding-system of the highest priority in the list.
+	  (let ((l detected-coding-system))
+	    (while (and l
+			(null (eq (get-code (car l)) coding)))
+	      (setq l (cdr l)))
+	    (setq coding-system (car (or l detected-coding-system))))))
+    ;; Then, decode the region.
+    (if coding-system
+	(save-restriction
+	  (narrow-to-region from to)
+	  (code-convert from to coding-system '*internal*)
+	  (goto-char (point-max))))))
+
 ;; the  rmail-break-forwarded-messages  feature is not implemented
 (defun rmail-convert-to-babyl-format ()
   (let ((count 0) start
