@@ -35,7 +35,7 @@ pre-composed Vietnamese characaters."
     (save-restriction
       (narrow-to-region from to)
       (goto-char (point-min))
-      (while (re-search-forward viqr-regexp nil t)
+      (while (re-search-forward viqr-regexp   0 t)
 	(setq key (buffer-substring (match-beginning 0) (match-end 0)))
 	(setq def (lookup-key map key))
 	(if (numberp def)
@@ -64,9 +64,9 @@ pre-composed Vietnamese characaters."
     (save-restriction
       (narrow-to-region from to)
       (goto-char (point-min))
-      (while (re-search-forward "\\cv" nil t)
-	(setq def (char-to-string (preceding-char)))
-	(if (setq key (assoc def decode-map))
+      (while (re-search-forward "\\cv"   0 t)
+	(setq def (preceding-char))
+	(if (setq key (assq def decode-map))
 	    (progn
 	      (delete-char -1)
 	      (insert (cdr key))))))))
