@@ -1,5 +1,5 @@
 /* Synchronous subprocess invocation for GNU Emacs.
-   Copyright (C) 1985, 86, 87, 88, 93, 94, 95 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1988, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* modified by A.Kato(kato@koudai.cs.titech.junet)  87.7.7 */
 /* 87.7.7   modified by K.Handa */
@@ -75,15 +74,6 @@ extern char *strerror ();
 #include <fcntl.h>
 #endif
 
-#ifdef WINDOWSNT
-#define NOMINMAX
-#include <windows.h>
-#include <stdlib.h>	/* for proper declaration of environ */
-#include <fcntl.h>
-#include "nt.h"
-#define _P_NOWAIT 1	/* from process.h */
-#endif
-
 #ifdef MSDOS	/* Demacs 1.1.1 91/10/16 HIRANO Satoshi */
 #include "msdos.h"
 #define INCLUDED_FCNTL
@@ -129,14 +119,14 @@ extern char **environ;
    LF <-> CRLF conversion accroding to `process-coding-system' variable.
    (by Mineharu Takahara mtakahar@jp.oracle.com)
    </MULE-COMMENT> */
-#if defined (DOS_NT) && !defined(MULE)
+#if defined(MSDOS) && !defined(MULE)
 /* When we are starting external processes we need to know whether they
    take binary input (no conversion) or text input (\n is converted to
    \r\n).  Similar for output: if newlines are written as \r\n then it's
    text process output, otherwise it's binary.  */
 Lisp_Object Vbinary_process_input;
 Lisp_Object Vbinary_process_output;
-#endif /* DOS_NT */
+#endif
 
 Lisp_Object Vexec_path, Vexec_directory, Vdata_directory, Vdoc_directory;
 Lisp_Object Vconfigure_info_directory;
@@ -150,9 +140,9 @@ Lisp_Object Vprocess_environment;
    LF <-> CRLF conversion accroding to `file-coding-system' variable.
    (by Mineharu Takahara mtakahar@jp.oracle.com)
    </MULE-COMMENT> */
-#if defined (DOS_NT) && !defined(MULE)
+#if defined(MSDOS) && !defined(MULE)
 Lisp_Object Qbuffer_file_type;
-#endif /* DOS_NT */
+#endif
 
 /* True iff we are about to fork off a synchronous process or if we
    are waiting for it.  */
@@ -234,18 +224,11 @@ DEFUN ("si:call-process", Fsi_call_process, Ssi_call_process, 1, MANY, 0,
 The program's input comes from file INFILE (nil means `/dev/null').\n\
 Insert output in BUFFER before point; t means current buffer;\n\
  nil for BUFFER means discard it; 0 means discard and don't wait.\n\
-BUFFER can also have the form (REAL-BUFFER STDERR-FILE); in that case,\n\
-REAL-BUFFER says what to do with standard output, as above,\n\
-while STDERR-FILE says what to do with standard error in the child.\n\
-STDERR-FILE may be nil (discard standard error output),\n\
-t (mix it with ordinary output), or a file name string.\n\
-\n\
 Fourth arg DISPLAY non-nil means redisplay buffer as output is inserted.\n\
 Fifth arg CODE specifies the coding-system object for input from PROCESS.\n\
 Remaining arguments are strings passed as command arguments to PROGRAM.\n\
-\n\
-If BUFFER is 0, `call-process' returns immediately with value nil.\n\
-Otherwise it waits for PROGRAM to terminate\n\
+If BUFFER is 0, returns immediately with value nil.\n\
+Otherwise waits for PROGRAM to terminate\n\
 and returns a numeric exit status or a signal description string.\n\
 If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   (nargs, args)
@@ -257,16 +240,11 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   int fd[2];
   int filefd;
   register int pid;
-  char buf[16384];
-  char *bufptr = buf;
-  int bufsize = 16384;
+  char buf[1024];
   int count = specpdl_ptr - specpdl;
   register unsigned char **new_argv /* 91.11.6 by K.Handa */
     = (unsigned char **) alloca ((max (2, nargs - 3)) * sizeof (char *));
   struct buffer *old = current_buffer;
-  /* File to use for stderr in the child.
-     t means use same as standard output.  */
-  Lisp_Object error_file;
 #ifdef MSDOS	/* Demacs 1.1.1 91/10/16 HIRANO Satoshi */
   char *outf, *tempfile;
   int outfilefd;
@@ -279,11 +257,9 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 
   CHECK_STRING (args[0], 0);
 
-  error_file = Qt;
-
 #ifndef subprocesses
   /* Without asynchronous processes we cannot have BUFFER == 0.  */
-  if (nargs >= 3 && INTEGERP (args[2]))
+  if (nargs >= 3 && XTYPE (args[2]) == Lisp_Int)
     error ("Operating system cannot handle asynchronous subprocesses");
 #endif /* subprocesses */
 
@@ -297,36 +273,14 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 
   if (nargs >= 3)
     {
-      buffer = args[2];
+      register Lisp_Object tem;
 
-      /* If BUFFER is a list, its meaning is
-	 (BUFFER-FOR-STDOUT FILE-FOR-STDERR).  */
-      if (CONSP (buffer))
+      buffer = tem = args[2];
+      if (!(EQ (tem, Qnil)
+	    || EQ (tem, Qt)
+	    || XFASTINT (tem) == 0))
 	{
-	  if (CONSP (XCONS (buffer)->cdr))
-	    {
-	      Lisp_Object stderr_file;
-	      stderr_file = XCONS (XCONS (buffer)->cdr)->car;
-
-	      if (NILP (stderr_file) || EQ (Qt, stderr_file))
-		error_file = stderr_file;
-	      else
-		error_file = Fexpand_file_name (stderr_file, Qnil);
-	    }
-
-	  buffer = XCONS (buffer)->car;
-	}
-
-      if (!(EQ (buffer, Qnil)
-	    || EQ (buffer, Qt)
-	    || XFASTINT (buffer) == 0))
-	{
-	  Lisp_Object spec_buffer;
-	  spec_buffer = buffer;
-	  buffer = Fget_buffer (buffer);
-	  /* Mention the buffer name for a better error message.  */
-	  if (NILP (buffer))
-	    CHECK_BUFFER (spec_buffer, 2);
+	  buffer = Fget_buffer (tem);
 	  CHECK_BUFFER (buffer, 2);
 	}
     }
@@ -380,7 +334,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
       close (filefd);
       report_file_error ("Searching for program", Fcons (args[0], Qnil));
     }
-  new_argv[0] = XSTRING (path)->data;
+
   {
     register int i;
     for (i = 5; i < nargs; i++)	/* 91.10.28 by K.Handa */
@@ -388,10 +342,20 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 	CHECK_STRING (args[i], i);
 	new_argv[i - 4] = XSTRING (args[i])->data; /* 91.10.28 by K.Handa */
       }
+    /* Program name is first command arg */
+    new_argv[0] = XSTRING (args[0])->data;
     new_argv[i - 4] = 0;	/* 91.10.28 by K.Handa */
   }
 
+  new_argv[0] = XSTRING (path)->data;
+
 #ifdef MSDOS /* MW, July 1993 */
+  /* These vars record information from process termination.
+     Clear them now before process can possibly terminate,
+     to avoid timing error if process terminates soon.  */
+  synch_process_death = 0;
+  synch_process_retcode = 0;
+
   if ((outf = egetenv ("TMP")) || (outf = egetenv ("TEMP")))
     strcpy (tempfile = alloca (strlen (outf) + 20), outf);
   else
@@ -409,14 +373,11 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   if (outfilefd < 0)
     {
       close (filefd);
-      report_file_error ("Opening process output file",
-			 Fcons (build_string (tempfile), Qnil));
+      report_file_error ("Opening process output file", Fcons (tempfile, Qnil));
     }
-  fd[0] = filefd;
-  fd[1] = outfilefd;
-#endif /* MSDOS */
+#endif
 
-  if (INTEGERP (buffer))
+  if (XTYPE (buffer) == Lisp_Int)
     fd[1] = open (NULL_DEVICE, O_WRONLY), fd[0] = -1;
   else
     {
@@ -428,13 +389,12 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
       set_exclusive_use (fd[0]);
 #endif
     }
-
+ 
   {
     /* child_setup must clobber environ in systems with true vfork.
        Protect it from permanent change.  */
     register char **save_environ = environ;
     register int fd1 = fd[1];
-    int fd_error = fd1;
 
 #if 0  /* Some systems don't have sigblock.  */
     mask = sigblock (sigmask (SIGCHLD));
@@ -449,54 +409,11 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
     synch_process_death = 0;
     synch_process_retcode = 0;
 
-    if (NILP (error_file))
-      fd_error = open (NULL_DEVICE, O_WRONLY);
-    else if (STRINGP (error_file))
-      {
-#ifdef DOS_NT
-	fd_error = open (XSTRING (error_file)->data,
-			 O_WRONLY | O_TRUNC | O_CREAT | O_TEXT,
-			 S_IREAD | S_IWRITE);
-#else  /* not DOS_NT */
-	fd_error = creat (XSTRING (error_file)->data, 0666);
-#endif /* not DOS_NT */
-      }
-
-    if (fd_error < 0)
-      {
-	close (filefd);
-	if (fd[0] != filefd)
-	  close (fd[0]);
-	if (fd1 >= 0)
-	  close (fd1);
-#ifdef MSDOS
-	unlink (tempfile);
-#endif
-	report_file_error ("Cannot redirect stderr",
-			   Fcons ((NILP (error_file)
-				   ? build_string (NULL_DEVICE) : error_file),
-				  Qnil));
-      }
 #ifdef MSDOS /* MW, July 1993 */
     /* ??? Someone who knows MSDOG needs to check whether this properly
-       closes all descriptors that it opens.
-
-       Note that run_msdos_command() actually returns the child process
-       exit status, not its PID, so we assign it to `synch_process_retcode'
-       below.  */
-    pid = run_msdos_command (new_argv, current_dir,
-			     filefd, outfilefd, fd_error);
-
-    /* Record that the synchronous process exited and note its
-       termination status.  */
-    synch_process_alive = 0;
-    synch_process_retcode = pid;
-    if (synch_process_retcode < 0)  /* means it couldn't be exec'ed */
-      synch_process_death = strerror(errno);
-
+       closes all descriptors that it opens.  */
+    pid = run_msdos_command (new_argv, current_dir, filefd, outfilefd);
     close (outfilefd);
-    if (fd_error != outfilefd)
-      close (fd_error);
     fd1 = -1; /* No harm in closing that one!  */
 #ifdef MULE
     fd[0] = open (tempfile, O_BINARY);
@@ -510,9 +427,6 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 	report_file_error ("Cannot re-open temporary file", Qnil);
       }
 #else /* not MSDOS */
-#ifdef WINDOWSNT
-    pid = child_setup (filefd, fd1, fd_error, new_argv, 0, current_dir);
-#else  /* not WINDOWSNT */
 #ifdef WIN32 /* 92.2.17 by M.Higashida */
     {
       HANDLE temp;
@@ -533,19 +447,14 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
       {
 	if (fd[0] >= 0)
 	  close (fd[0]);
-#if defined(USG) && !defined(BSD_PGRPS)
+#ifdef USG
         setpgrp ();
 #else
         setpgrp (pid, pid);
 #endif /* USG */
-	child_setup (filefd, fd1, fd_error, new_argv, 0, current_dir);
+	child_setup (filefd, fd1, fd1, new_argv, 0, current_dir);
       }
 #endif /* not WIN32 */
-#endif /* not WINDOWSNT */
-
-    /* The MSDOS case did this already.  */
-    if (fd_error >= 0)
-      close (fd_error);
 #endif /* not MSDOS */
 
     environ = save_environ;
@@ -554,10 +463,10 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
        since we will use that to read input from.  */
     close (filefd);
 #ifdef WIN32 /* 93.2.25 by M.Higashida */
-    if (fd1 >= 0 && fd1 != fd_error)
+    if (fd1 >= 0)
       CloseHandle ((HANDLE) fd1);
 #else /* not WIN32 */
-    if (fd1 >= 0 && fd1 != fd_error)
+    if (fd1 >= 0)
       close (fd1);
 #endif /* not WIN32 */
   }
@@ -571,7 +480,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
     }
 #endif /* not WIN32 */
 
-  if (INTEGERP (buffer))
+  if (XTYPE (buffer) == Lisp_Int)
     {
       if (fd[0] >= 0)
 	close (fd[0]);
@@ -597,7 +506,7 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 #endif /* not MSDOS */
 
 
-  if (BUFFERP (buffer))
+  if (XTYPE (buffer) == Lisp_Buffer)
     Fset_buffer (buffer);
 
   immediate_quit = 1;
@@ -609,83 +518,49 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 #else /* not WIN32 */
     register int nread;
 #endif /* not WIN32 */
+    register int state = 1;
     int first = 1;
-    int total_read = 0;
+    char *buf2;
 
-    while (1)
+    while (state)
       {
-	/* Repeatedly read until we've filled as much as possible
-	   of the buffer size we have.  But don't read
-	   less than 1024--save that for the next bufferful.  */
-
-	nread = 0;
-	while (nread < bufsize - 1024)
-	  {
-	    int this_read;
-
 /* 91.10.28 by M.Higashida */
 #ifdef WIN32 /* 93.2.25 by M.Higashida */
-	    ReadFile ((HANDLE) fd[0], bufptr + nread, bufsize - nread,
-		      &this_read, 0);
+	ReadFile ((HANDLE) fd[0], buf, sizeof buf, &nread, 0);
 #else /* not WIN32 */
 #if defined(MSDOS) && defined(GO32)
-	    this_read = (this_read = read (fd[0], bufptr + nread,
-					   bufsize - nread)) ? this_read : -1;
+	nread = (nread = read (fd[0], buf, sizeof buf)) ? nread : -1;
 #else
-	    this_read = read (fd[0], bufptr + nread, bufsize - nread);
+	nread = read (fd[0], buf, sizeof buf);
 #endif
 #endif /* not WIN32 */
+	if (nread <= 0) state = 0;
 /* end of patch */
-
-	    if (this_read < 0)
-	      goto give_up;
-
-	    if (this_read == 0)
-	      goto give_up_1;
-
-	    nread += this_read;
-	  }
-
-      give_up_1:
-
 	immediate_quit = 0;
-	total_read += nread;
-	
 	if (!NILP (buffer))
 	  {			/* 91.10.28 by K.Handa */
-	    char *p;
-	    int n;
-
 	    if ((CODE_TYPE(&mccode) == AUTOCONV)
 		|| (CODE_TYPE(&mccode) > ITNCODE))
 	      {
 		/* 93.2.10, 94.6.22 by K.Handa */
-		p = get_conversion_buffer (ENCODE_BUF_SIZE (nread, &mccode));
-		if (nread == 0)
-		  CODE_CNTL(&mccode) |= CC_END;
-		n = encode (&mccode, bufptr, p, nread, &dummy);
+		if (state)
+		  buf2 =
+		    get_conversion_buffer(ENCODE_BUF_SIZE (nread, &mccode));
+		else
+		  {
+		    nread = 0;
+		    buf2 = get_conversion_buffer(ENCODE_BUF_SIZE (0, &mccode));
+		    CODE_CNTL(&mccode) |= CC_END;
+		  }
+		if (nread = encode(&mccode, buf, buf2, nread, &dummy))
+		  insert (buf2, nread);
 	      } 
 	    else
 	      {
-		p = bufptr;
-		n = nread;
+		if (state)
+		  insert (buf, nread);
 	      }
-
-	    insert (p, n);
 	  }
-
-	/* Now NREAD is the total amount of data in the buffer.  */
-	if (nread == 0)
-	  break;
-
-	/* Make the buffer bigger as we continue to read more data,
-	   but not past 64k.  */
-	if (bufsize < 64 * 1024 && total_read > 32 * bufsize)
-	  {
-	    bufsize *= 2;
-	    bufptr = (char *) alloca (bufsize);
-	  }
-
 	if (!NILP (display) && INTERACTIVE)
 	  {
 	    if (first)
@@ -696,7 +571,6 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
 	immediate_quit = 1;
 	QUIT;
       }
-  give_up: ;
   }
 
   /* Wait for it to terminate, unless it already has.  */
@@ -733,29 +607,19 @@ static Lisp_Object
 delete_temp_file (name)
      Lisp_Object name;
 {
-  /* Use Fdelete_file (indirectly) because that runs a file name handler.
-     We did that when writing the file, so we should do so when deleting.  */
-  internal_delete_file (name);
+  unlink (XSTRING (name)->data);
 }
 
 DEFUN ("call-process-region", Fcall_process_region, Scall_process_region,
   3, MANY, 0,
   "Send text from START to END to a synchronous process running PROGRAM.\n\
 Delete the text if fourth arg DELETE is non-nil.\n\
-\n\
 Insert output in BUFFER before point; t means current buffer;\n\
  nil for BUFFER means discard it; 0 means discard and don't wait.\n\
-BUFFER can also have the form (REAL-BUFFER STDERR-FILE); in that case,\n\
-REAL-BUFFER says what to do with standard output, as above,\n\
-while STDERR-FILE says what to do with standard error in the child.\n\
-STDERR-FILE may be nil (discard standard error output),\n\
-t (mix it with ordinary output), or a file name string.\n\
-\n\
 Sixth arg DISPLAY non-nil means redisplay buffer as output is inserted.\n\
 Remaining args are passed to PROGRAM at startup as command args.\n\
-\n\
-If BUFFER is nil, `call-process-region' returns immediately with value nil.\n\
-Otherwise it waits for PROGRAM to terminate\n\
+If BUFFER is nil, returns immediately with value nil.\n\
+Otherwise waits for PROGRAM to terminate\n\
 and returns a numeric exit status or a signal description string.\n\
 If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   (nargs, args)
@@ -765,13 +629,13 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   struct gcpro gcpro1;
   Lisp_Object filename_string;
   register Lisp_Object start, end;
-#ifdef DOS_NT
+#ifdef MSDOS
   char *tempfile;
 #else
   char tempfile[20];
 #endif
   int count = specpdl_ptr - specpdl;
-#ifdef DOS_NT
+#ifdef MSDOS
   char *outf = '\0';
 
   if ((outf = egetenv ("TMP")) || (outf = egetenv ("TEMP")))
@@ -781,25 +645,19 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
       tempfile = alloca (20);
       *tempfile = '\0';
     }
-  if (!IS_DIRECTORY_SEP (tempfile[strlen (tempfile) - 1]))
+  dostounix_filename (tempfile);
+  if (tempfile[strlen (tempfile) - 1] != '/')
     strcat (tempfile, "/");
-  if ('/' == DIRECTORY_SEP)
-    dostounix_filename (tempfile);
-  else
-    unixtodos_filename (tempfile);
-#ifdef WINDOWSNT
-  strcat (tempfile, "emXXXXXX");
-#else
   strcat (tempfile, "detmp.XXX");
-#endif
-#else /* not DOS_NT */
+#else /* not MSDOS */
 
 #ifdef VMS
   strcpy (tempfile, "tmp:emacsXXXXXX.");
 #else
   strcpy (tempfile, "/tmp/emacsXXXXXX");
 #endif
-#endif /* not DOS_NT */
+#endif /* not MSDOS */
+/* end of patch */
 
   mktemp (tempfile);
 
@@ -807,13 +665,13 @@ If you quit, the process is killed with SIGINT, or SIGKILL if you quit again.")
   GCPRO1 (filename_string);
   start = args[0];
   end = args[1];
-#ifdef DOS_NT && !defined(MULE)
+#if defined(MSDOS) && !defined(MULE)
   specbind (Qbuffer_file_type, Vbinary_process_input);
-  Fwrite_region (start, end, filename_string, Qnil, Qlambda, Qnil);
+  Fwrite_region (start, end, filename_string, Qnil, Qlambda);
   unbind_to (count, Qnil);
-#else  /* not DOS_NT */
-  Fwrite_region (start, end, filename_string, Qnil, Qlambda, Qnil);
-#endif /* not DOS_NT */
+#else
+  Fwrite_region (start, end, filename_string, Qnil, Qlambda);
+#endif
 
   record_unwind_protect (delete_temp_file, filename_string);
 
@@ -894,7 +752,7 @@ child_setup (in, out, err, new_argv, env)
     
   {
     /* change directory */
-    if (STRINGP (current_buffer->directory))
+    if (XTYPE (current_buffer->directory) == Lisp_String)
       {
 	register unsigned char *temp;
 	register int i;
@@ -908,7 +766,7 @@ child_setup (in, out, err, new_argv, env)
       }
   }
 #else
-  if (STRINGP (current_buffer->directory))
+  if (XTYPE (current_buffer->directory) == Lisp_String)
     {
       register int i;
       
@@ -992,10 +850,6 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
 #else /* not MSDOS */
   char **env;
   char *pwd_var;
-#ifdef WINDOWSNT
-  int cpid;
-  HANDLE handles[3];
-#endif /* WINDOWSNT */
 
   int pid = getpid ();
 
@@ -1028,7 +882,7 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
     temp = pwd_var + 4;
     bcopy ("PWD=", pwd_var, 4);
     bcopy (XSTRING (current_dir)->data, temp, i);
-    if (!IS_DIRECTORY_SEP (temp[i - 1])) temp[i++] = DIRECTORY_SEP;
+    if (temp[i - 1] != '/') temp[i++] = '/';
     temp[i] = 0;
 
     /* We can't signal an Elisp error here; we're in a vfork.  Since
@@ -1037,10 +891,10 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
        are changed between the check and this chdir, but we should
        at least check.  */
     if (chdir (temp) < 0)
-      _exit (errno);
+      exit (errno);
 
     /* Strip trailing slashes for PWD, but leave "/" and "//" alone.  */
-    while (i > 2 && IS_DIRECTORY_SEP (temp[i - 1]))
+    while (i > 2 && temp[i - 1] == '/')
       temp[--i] = 0;
   }
 
@@ -1052,7 +906,8 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
 
     new_length = 0;
     for (tem = Vprocess_environment;
-	 CONSP (tem) && STRINGP (XCONS (tem)->car);
+	 (XTYPE (tem) == Lisp_Cons
+	  && XTYPE (XCONS (tem)->car) == Lisp_String);
 	 tem = XCONS (tem)->cdr)
       new_length++;
 
@@ -1066,7 +921,8 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
 
     /* Copy the Vprocess_environment strings into new_env.  */
     for (tem = Vprocess_environment;
-	 CONSP (tem) && STRINGP (XCONS (tem)->car);
+	 (XTYPE (tem) == Lisp_Cons
+	  && XTYPE (XCONS (tem)->car) == Lisp_String);
 	 tem = XCONS (tem)->cdr)
       {
 	char **ep = env;
@@ -1095,32 +951,19 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
       }
     *new_env = 0;
   }
-#ifdef WINDOWSNT
-  prepare_standard_handles (in, out, err, handles);
-#else  /* not WINDOWSNT */
+
   /* Make sure that in, out, and err are not actually already in
      descriptors zero, one, or two; this could happen if Emacs is
      started with its standard in, out, or error closed, as might
      happen under X.  */
-  {
-    int oin = in, oout = out;
-
-    /* We have to avoid relocating the same descriptor twice!  */
-
-    in = relocate_fd (in, 3);
-
-    if (out == oin)
-      out = in;
-    else
+  in = relocate_fd (in, 3);
+  if (out == err)
+    err = out = relocate_fd (out, 3);
+  else
+    {
       out = relocate_fd (out, 3);
-
-    if (err == oin)
-      err = in;
-    else if (err == oout)
-      err = out;
-    else
       err = relocate_fd (err, 3);
-  }
+    }
 
   close (0);
   close (1);
@@ -1132,9 +975,8 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
   close (in);
   close (out);
   close (err);
-#endif /* not WINDOWSNT */
 
-#if defined(USG) && !defined(BSD_PGRPS)
+#ifdef USG
 #ifndef SETPGRP_RELEASES_CTTY
   setpgrp ();			/* No arguments but equivalent in this case */
 #endif
@@ -1148,26 +990,15 @@ child_setup (in, out, err, new_argv, set_pgrp, current_dir)
   something missing here;
 #endif /* vipc */
 
-#ifdef WINDOWSNT
-  /* Spawn the child.  (See ntproc.c:Spawnve).  */
-  cpid = spawnve (_P_NOWAIT, new_argv[0], new_argv, env);
-  if (cpid == -1)
-    /* An error occurred while trying to spawn the process.  */
-    report_file_error ("Spawning child process", Qnil);
-  reset_standard_handles (in, out, err, handles);
-  return cpid;
-#else /* not WINDOWSNT */
   /* execvp does not accept an environment arg so the only way
      to pass this environment is to set environ.  Our caller
      is responsible for restoring the ambient value of environ.  */
   environ = env;
   execvp (new_argv[0], new_argv);
 
-  write (1, "Can't exec program: ", 20);
+  write (1, "Couldn't exec the program ", 26);
   write (1, new_argv[0], strlen (new_argv[0]));
-  write (1, "\n", 1);
   _exit (1);
-#endif /* not WINDOWSNT */
 #endif /* not MSDOS */
 }
 
@@ -1217,16 +1048,10 @@ getenv_internal (var, varlen, value, valuelen)
       Lisp_Object entry;
 
       entry = XCONS (scan)->car;
-      if (STRINGP (entry)
+      if (XTYPE (entry) == Lisp_String
 	  && XSTRING (entry)->size > varlen
 	  && XSTRING (entry)->data[varlen] == '='
-#ifdef WINDOWSNT
-	  /* NT environment variables are case insensitive.  */
-	  && ! strnicmp (XSTRING (entry)->data, var, varlen)
-#else  /* not WINDOWSNT */
-	  && ! bcmp (XSTRING (entry)->data, var, varlen)
-#endif /* not WINDOWSNT */
-	  )
+	  && ! bcmp (XSTRING (entry)->data, var, varlen))
 	{
 	  *value    = (char *) XSTRING (entry)->data + (varlen + 1);
 	  *valuelen = XSTRING (entry)->size - (varlen + 1);
@@ -1293,7 +1118,7 @@ init_callproc_1 ()
   Vexec_path = nconc2 (decode_env_path ("PATH", ""), Vexec_path);
 }
 
-/* This is run after init_cmdargs, when Vinstallation_directory is valid.  */
+/* This is run after init_cmdargs, so that Vinvocation_directory is valid.  */
 
 init_callproc ()
 {
@@ -1310,25 +1135,26 @@ init_callproc ()
 			       Vinstallation_directory);
       if (NILP (Fmember (tem, Vexec_path)))
 	{
-#ifndef DOS_NT
+#ifndef MSDOS
 	  /* MSDOS uses wrapped binaries, so don't do this.  */
 	  Vexec_path = nconc2 (Vexec_path, Fcons (tem, Qnil));
 	  Vexec_directory = Ffile_name_as_directory (tem);
-#endif /* not DOS_NT */
-	}
+#endif
 
-      /* Maybe use ../etc as well as ../lib-src.  */
-      if (data_dir == 0)
-	{
-	  tem = Fexpand_file_name (build_string ("etc"),
-				   Vinstallation_directory);
-	  Vdoc_directory = Ffile_name_as_directory (tem);
+	  /* If we use ../lib-src, maybe use ../etc as well.
+	     Do so if ../etc exists and has our DOC-... file in it.  */
+	  if (data_dir == 0)
+	    {
+	      tem = Fexpand_file_name (build_string ("etc"),
+				       Vinstallation_directory);
+	      Vdoc_directory = Ffile_name_as_directory (tem);
+	    }
 	}
     }
 
   /* Look for the files that should be in etc.  We don't use
      Vinstallation_directory, because these files are never installed
-     near the executable, and they are never in the build
+     in /bin near the executable, and they are never in the build
      directory when that's different from the source directory.
 
      Instead, if these files are not in the nominal place, we try the
@@ -1412,7 +1238,7 @@ syms_of_callproc ()
    `file-coding-system' variable.  (by Mineharu Takahara
    mtakahar@jp.oracle.com)
    </MULE-COMMENT> */
-#if defined (DOS_NT) && !defined(MULE)
+#if defined(MSDOS) && !defined(MULE)
   Qbuffer_file_type = intern ("buffer-file-type");
   staticpro (&Qbuffer_file_type);
 
@@ -1423,7 +1249,7 @@ syms_of_callproc ()
   DEFVAR_LISP ("binary-process-output", &Vbinary_process_output,
     "*If non-nil then new subprocesses are assumed to produce binary output.");
   Vbinary_process_output = Qnil;
-#endif /* DOS_NT */
+#endif
 
   DEFVAR_LISP ("shell-file-name", &Vshell_file_name,
     "*File name to load inferior shells from.\n\
