@@ -54,8 +54,8 @@ is a vetor of 127 elements, which are description strings for each category.")
   (obj)
      Lisp_Object obj;
 {
-  if (VECTORP (obj) && XVECTOR (obj)->size == 257
-      && VECTORP (XVECTOR (obj)->contents[256])
+  if (XTYPE (obj) == Lisp_Vector && XVECTOR (obj)->size == 257
+      && XTYPE (XVECTOR (obj)->contents[256]) == Lisp_Vector
       && XVECTOR (XVECTOR (obj)->contents[256])->size == 95)
     return Qt;
   return Qnil;
@@ -101,7 +101,11 @@ init_category(category)
 Lisp_Object
 make_init_category()
 {
-  return Fmake_string (make_number (12), make_number (0));
+  register Lisp_Object category;
+  Lisp_Object size, init;
+
+  XFASTINT (size) = 12; XFASTINT (init) = 0;
+  return Fmake_string (size, init);
 }
 
 update_category(src, mask)
@@ -118,8 +122,8 @@ DEFUN ("category-equal", Fcategory_equal, Scategory_equal, 2, 2, 0,
   (c1, c2)
      register Lisp_Object c1, c2;
 {
-  if (!STRINGP (c1)
-      || !STRINGP (c2)
+  if (XTYPE (c1) != Lisp_String
+      || XTYPE (c2) != Lisp_String
       || XCATEGORY (c1)->data[0] != XCATEGORY (c2)->data[0]
       || XCATEGORY (c1)->data[1] != XCATEGORY (c2)->data[1]
       || XCATEGORY (c1)->data[2] != XCATEGORY (c2)->data[2])
@@ -199,10 +203,10 @@ make_init_category_table(size)
   Lisp_Object val;
   int i = XFASTINT (size);
 
-  XSETFASTINT (val, 0);
+  XFASTINT (val) = 0;
   val = Fmake_vector (size, val);
   if (i == 257) {
-    XSETFASTINT (size, 95);
+    XFASTINT (size) = 95;
     XVECTOR (val)->contents[256] = Fmake_vector (size, Qnil);
     for (i = 0; i < 95; i++)
       XVECTOR (XVECTOR (val)->contents[256])->contents[i] = Qnil;
@@ -223,7 +227,7 @@ copy_category_table(table)
   bcopy (XVECTOR (table)->contents,
 	 XVECTOR (val)->contents, 256 * sizeof (Lisp_Object));
   for (i = 0; i < 256; i++) {
-    if (CONSP (XVECTOR (table)->contents[i])) {
+    if (XTYPE (XVECTOR (table)->contents[i]) == Lisp_Cons) {
       temp = XVECTOR (table)->contents[i];
       XSET (temp, Lisp_Cons,
 	    Fcons (Fcar (temp), copy_category_table(Fcdr (temp))));
@@ -299,7 +303,7 @@ check_category_at(p, len, ctbl, mnemonic, not)
       if (c == 0x80)
 	len--, c = *p++ & 0x7F;
     }
-    if (CONSP ((temp = XVECTOR (ctbl)->contents[c]))) {
+    if (XTYPE ((temp = XVECTOR (ctbl)->contents[c])) == Lisp_Cons) {
       if (XCATEGORY (XCONS (temp)->car)->data[m1] & m2) return !not;
       ctbl = XCONS (temp)->cdr;
     } else {
@@ -355,7 +359,7 @@ char_category(c, ctbl)
 
   init_category(&temp_category);
 
-  while (i > 1 && CONSP (XVECTOR (ctbl)->contents[*p])) {
+  while (i > 1 && XTYPE (XVECTOR (ctbl)->contents[*p]) == Lisp_Cons) {
     update_category (&temp_category,
 		     XCONS (XVECTOR (ctbl)->contents[*p])->car);
     ctbl = XCONS (XVECTOR (ctbl)->contents[*p])->cdr;
@@ -412,7 +416,7 @@ modify_category_entry(c, maskbit, ctbl, reset)
   register int i = CHARtoSTR (c, str);
 
   while (i-- > 1) {
-    if (!CONSP (XVECTOR (ctbl)->contents[*p]))
+    if (XTYPE (XVECTOR (ctbl)->contents[*p]) != Lisp_Cons)
       XVECTOR (ctbl)->contents[*p] =
 	Fcons (XVECTOR (ctbl)->contents[*p],
 	       make_init_category_table (256));
@@ -420,7 +424,7 @@ modify_category_entry(c, maskbit, ctbl, reset)
     p++;
   }
   obj = XVECTOR (ctbl)->contents[*p];
-  if (CONSP (obj)) obj = XCONS (obj)->car;
+  if (XTYPE (obj) == Lisp_Cons) obj = XCONS (obj)->car;
   
   if (reset) XCATEGORY (obj)->data[maskbit / 32] &= ~(1 << (maskbit % 32));
   else XCATEGORY (obj)->data[maskbit / 32] |= 1 << (maskbit % 32);
@@ -516,12 +520,12 @@ describe_category (ctbl, parent) /* 94.2.23 by K.Handae */
     Findent_to (make_number (24), make_number (1));
     
     deeper = 0;
-    if (CONSP (category)) {
+    if (XTYPE (category) == Lisp_Cons) {
       category = XCONS (category)->car;
       deeper = 1;
     }
 
-    if (!STRINGP (category) || XSTRING (category)->size != 12) {
+    if (XTYPE (category) != Lisp_String || XSTRING (category)->size != 12) {
       insert_string ("invalid");
     } else {
       pack_mnemonic_string (XCATEGORY (category), str);
