@@ -72,16 +72,17 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
 #ifdef emacs
 #include <sys/types.h>
 #include <config.h>
+#else  /* not emacs */
+#define xmalloc (void *)malloc
+#define xrealloc (void *)realloc
+#endif /* emacs */
 
+#ifdef emacs
 #include "lisp.h"
 #include "buffer.h"
 #include "charset.h"		/* 92.1.21 by K.Handa */
 #else  /* not emacs */
 #include "mulelib.h"
-				/* Define these macros after loading
-                                   mulelib.h for alpha system.  */
-#define xmalloc (void *)malloc
-#define xrealloc (void *)realloc
 #endif /* not emacs */
 
 #ifdef emacs
@@ -306,8 +307,8 @@ Optional arg DIRECTION specifies writing direction (0:normal, 1:r2l).")
 
   CHECK_NUMBER (type, 0);
   CHECK_NUMBER (final, 0);
-  XSETFASTINT (val, lc_table[XFASTINT (type) & 3][XFASTINT (final) & 0xFF]);
-  if (dir) XSETFASTINT (val, rev_lc_table[XFASTINT (val)]);
+  XFASTINT (val) = lc_table[XFASTINT (type) & 3][XFASTINT (final) & 0xFF];
+  if (dir) XFASTINT (val) = rev_lc_table[XFASTINT (val)];
   return val;
 }
 
@@ -415,7 +416,7 @@ If the character does not have the componets, 0 is returned.")
     error ("Invalid index: %d", i);
 
   if (! MC_CHAR_P (c))		/* Type 1-1 */
-    XSETFASTINT (val, (i == 0)? c : 0);
+    XFASTINT (val) = (i == 0)? c : 0;
   else
     {
       unsigned char buffer[3], *s = buffer;
@@ -427,7 +428,7 @@ If the character does not have the componets, 0 is returned.")
 	*s++ = CHAR_HIGH (c);
       *s++ = CHAR_LOW (c);
 
-      XSETFASTINT (val, (buffer[i] > 0x80)? buffer[i] : 0);
+      XFASTINT (val) = (buffer[i] > 0x80)? buffer[i] : 0;
     }
 
   return val;
@@ -646,7 +647,7 @@ Each multilingual/composite character is also counted as one.")
     p++;
   }
 
-  XSETFASTINT (val, i);
+  XFASTINT (val) = i;
   return val;
 }
 
