@@ -166,6 +166,34 @@ forward_point(n)
 }
 /* end of patch */
 
+DEFUN ("point+", Fpoint_plus, Spoint_plus, 1, 1, 0,
+  "Return the position of next ARG character.")
+  (n)
+     Lisp_Object n;
+{
+  int pos;
+
+  CHECK_NUMBER (n, 0);
+  pos = forward_point (XINT (n));
+  if (pos < BEGV) pos = BEGV;
+  else if (pos > ZV) pos = ZV;
+  return make_number (pos);
+}
+
+DEFUN ("point-", Fpoint_minus, Spoint_minus, 1, 1, 0,
+  "Return the position of previoius ARG character.")
+  (n)
+     Lisp_Object n;
+{
+  int pos;
+
+  CHECK_NUMBER (n, 0);
+  pos = forward_point (- XINT (n));
+  if (pos < BEGV) pos = BEGV;
+  else if (pos > ZV) pos = ZV;
+  return make_number (pos);
+}
+
 DEFUN ("forward-line", Fforward_line, Sforward_line, 0, 1, "p",
   "Move ARG lines forward (backward if ARG is negative).\n\
 Precisely, if point is on line I, move to the start of line I + ARG.\n\
@@ -501,6 +529,8 @@ More precisely, a char with closeparen syntax is self-inserted.");
   defsubr (&Sbackward_char);
   defsubr (&Sforward_byte);	/* 92.4.1 by K.Handa */
   defsubr (&Sbackward_byte);	/* 92.4.1 by K.Handa */
+  defsubr (&Spoint_plus);
+  defsubr (&Spoint_minus);
   defsubr (&Sforward_line);
   defsubr (&Sbeginning_of_line);
   defsubr (&Send_of_line);
