@@ -454,7 +454,7 @@ subsidieary coding-system accoding to a detected end-of-line type.")
       if (mask & (1 << code_priority_category[i])) {
 	val2 = Fsymbol_value (code_category[code_priority_category[i]]);
 	val3 = Fget (val2, Qeol_type);
-	if (eol != CODE_EOL_AUTO && VECTORP (val3)) {
+	if (eol != CODE_EOL_AUTO && XTYPE (val3) == Lisp_Vector) {
 	  switch (eol) {
 	  case CODE_EOL_LF:
 	    val2 = XVECTOR (val3)->contents[0]; break;
@@ -1332,15 +1332,15 @@ DEFUN ("coding-system-p", Fcoding_system_p, Scoding_system_p, 1, 1, 0,
 {
   if (NILP (obj))
     return Qt;
-  while (!NILP (obj) && SYMBOLP (obj))
+  while (!NILP (obj) && XTYPE (obj) == Lisp_Symbol)
     obj = Fget (obj, Qcoding_system);
   if (NILP (obj))
     return Qnil;
 
-  if (VECTORP (obj)
+  if (XTYPE (obj) == Lisp_Vector
       && XVECTOR (obj)->size == 5
       && (XFASTINT (XVECTOR (obj)->contents[0]) != 2
-	  || (VECTORP (XVECTOR (obj)->contents[4])
+	  || (XTYPE (XVECTOR (obj)->contents[4]) == Lisp_Vector
 	      && XVECTOR (XVECTOR (obj)->contents[4])->size == 32)))
     return Qt;
   return Qnil;
@@ -1408,14 +1408,14 @@ encode_code(code, mccode)
   CODE_FORM (mccode) = 0;
 
   prop = code;
-  while (!NILP (prop) && SYMBOLP (prop))
+  while (!NILP (prop) && XTYPE (prop) == Lisp_Symbol)
     prop = Fget (prop, Qcoding_system);
 
-  if (VECTORP (prop)
+  if (XTYPE (prop) == Lisp_Vector
       && XVECTOR (prop)->size == 5) {
     type = XVECTOR (prop)->contents[0];
     eol_type = Fget(code, Qeol_type);
-    if (VECTORP (eol_type)) { /* 94.4.2 by K.Handa */
+    if (XTYPE (eol_type) == Lisp_Vector) { /* 94.4.2 by K.Handa */
       CODE_LF (mccode) = XVECTOR (eol_type)->contents[0];
       CODE_CRLF (mccode) = XVECTOR (eol_type)->contents[1];
       CODE_CR (mccode) = XVECTOR (eol_type)->contents[2];
@@ -1437,7 +1437,7 @@ encode_code(code, mccode)
       break;
     case 2:
       prop = XVECTOR (prop)->contents[4];
-      if (!VECTORP (prop) || XVECTOR (prop)->size != 32)
+      if (XTYPE (prop) != Lisp_Vector || XVECTOR (prop)->size != 32)
 	break;
       CODE_TYPE_SET (mccode, ISO2022);
       flags = XVECTOR (prop)->contents;
@@ -1455,7 +1455,9 @@ encode_code(code, mccode)
       break;
     case 4:
       prop = XVECTOR (prop)->contents[4];
-      if (CONSP (prop) && VECTORP (Fcar (prop)) && VECTORP (Fcdr (prop))) {
+      if (XTYPE (prop) == Lisp_Cons
+	  && XTYPE (Fcar (prop)) == Lisp_Vector
+	  && XTYPE (Fcdr (prop)) == Lisp_Vector) {
 	CODE_TYPE_SET (mccode, CCL);
 	set_ccl_program (&CODE_CCL_ENCODE (mccode), Fcar (prop));
 	set_ccl_program (&CODE_CCL_DECODE (mccode), Fcdr (prop));
@@ -1478,11 +1480,11 @@ and return cons of them.")
   (c1, c2)
      Lisp_Object c1, c2;
 {
-  int e1, e2;
+  Lisp_Object e1, e2;
 
   CHECK_NUMBER (c1, 0); CHECK_NUMBER (c2, 1);
-  S2E (XFASTINT (c1), XFASTINT (c2), e1, e2);
-  return Fcons (XFASTINT (e1), XFASTINT (e2));
+  S2E (XFASTINT (c1), XFASTINT (c2), XFASTINT (e1), XFASTINT (e2));
+  return Fcons (e1, e2);
 }
 
 DEFUN ("e2s", Fe2s, Se2s, 2, 2, 0,
@@ -1491,11 +1493,11 @@ and return cons of them.")
   (e1, e2)
      Lisp_Object e1, e2;
 {
-  int c1, c2;
+  Lisp_Object c1, c2;
 
   CHECK_NUMBER (e1, 0); CHECK_NUMBER (e2, 1);
-  E2S (XFASTINT (e1), XFASTINT (e2), c1, c2);
-  return Fcons (XFASTINT (c1), XFASTINT (c2));
+  E2S (XFASTINT (e1), XFASTINT (e2), XFASTINT (c1), XFASTINT (c2));
+  return Fcons (c1, c2);
 }
 
 /* 92.3.24 by K.Handa - make it non-interactive */
