@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 1, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 
 #include <config.h>
@@ -83,8 +82,8 @@ extern struct direct *readdir ();
 
 #include "regex.h"
 
-/* Returns a search buffer, with a fastmap allocated and ready to go.  */
-extern struct re_pattern_buffer *compile_pattern ();
+/* A search buffer, with a fastmap allocated and ready to go.  */
+extern struct re_pattern_buffer searchbuf;
 
 #define min(a, b) ((a) < (b) ? (a) : (b))
 
@@ -108,30 +107,28 @@ Lisp_Object Qfile_attributes;
 DEFUN ("directory-files", Fdirectory_files, Sdirectory_files, 1, 4, 0,
   "Return a list of names of files in DIRECTORY.\n\
 There are three optional arguments:\n\
-If FULL is non-nil, return absolute file names.  Otherwise return names\n\
- that are relative to the specified directory.\n\
-If MATCH is non-nil, mention only file names that match the regexp MATCH.\n\
+If FULL is non-nil, absolute pathnames of the files are returned.\n\
+If MATCH is non-nil, only pathnames containing that regexp are returned.\n\
 If NOSORT is non-nil, the list is not sorted--its order is unpredictable.\n\
  NOSORT is useful if you plan to sort the result yourself.")
-  (directory, full, match, nosort)
-     Lisp_Object directory, full, match, nosort;
+  (dirname, full, match, nosort)
+     Lisp_Object dirname, full, match, nosort;
 {
   DIR *d;
-  int dirnamelen;
+  int length;
   Lisp_Object list, name, dirfilename;
   Lisp_Object handler;
-  struct re_pattern_buffer *bufp;
 
   /* If the file name has special constructs in it,
      call the corresponding file handler.  */
-  handler = Ffind_file_name_handler (directory, Qdirectory_files);
+  handler = Ffind_file_name_handler (dirname, Qdirectory_files);
   if (!NILP (handler))
     {
       Lisp_Object args[6];
 
       args[0] = handler;
       args[1] = Qdirectory_files;
-      args[2] = directory;
+      args[2] = dirname;
       args[3] = full;
       args[4] = match;
       args[5] = nosort;
@@ -144,10 +141,10 @@ If NOSORT is non-nil, the list is not sorted--its order is unpredictable.\n\
     /* Because of file name handlers, these functions might call
      Ffuncall, and cause a GC.  */
     GCPRO1 (match);
-    directory = Fexpand_file_name (directory, Qnil);
+    dirname = Fexpand_file_name (dirname, Qnil);
     UNGCPRO;
-    GCPRO2 (match, directory);
-    dirfilename = Fdirectory_file_name (directory);
+    GCPRO2 (match, dirname);
+    dirfilename = Fdirectory_file_name (dirname);
     UNGCPRO;
   }
 
@@ -156,29 +153,29 @@ If NOSORT is non-nil, the list is not sorted--its order is unpredictable.\n\
       CHECK_STRING (match, 3);
 
       /* MATCH might be a flawed regular expression.  Rather than
-	 catching and signaling our own errors, we just call
+	 catching and signalling our own errors, we just call
 	 compile_pattern to do the work for us.  */
 #ifdef VMS
-      bufp = compile_pattern (match, 0,
-			      buffer_defaults.downcase_table->contents, 0);
+      compile_pattern (match, &searchbuf, 0,
+		       buffer_defaults.downcase_table->contents);
 #else
-      bufp = compile_pattern (match, 0, 0, 0);
+      compile_pattern (match, &searchbuf, 0, 0);
 #endif
     }
 
-  /* Now *bufp is the compiled form of MATCH; don't call anything
+  /* Now searchbuf is the compiled form of MATCH; don't call anything
      which might compile a new regexp until we're done with the loop!  */
 
   /* Do this opendir after anything which might signal an error; if
-     an error is signaled while the directory stream is open, we
+     an error is signalled while the directory stream is open, we
      have to make sure it gets closed, and setting up an
      unwind_protect to do so would be a pain.  */
   d = opendir (XSTRING (dirfilename)->data);
   if (! d)
-    report_file_error ("Opening directory", Fcons (directory, Qnil));
+    report_file_error ("Opening directory", Fcons (dirname, Qnil));
 
   list = Qnil;
-  dirnamelen = XSTRING (directory)->size;
+  length = XSTRING (dirname)->size;
 
   /* Loop reading blocks */
   while (1)
@@ -191,28 +188,27 @@ If NOSORT is non-nil, the list is not sorted--its order is unpredictable.\n\
       if (DIRENTRY_NONEMPTY (dp))
 	{
 	  if (NILP (match)
-	      || (0 <= re_search (bufp, dp->d_name, len, 0, len, 0)))
+	      || (0 <= re_search (&searchbuf, dp->d_name, len, 0, len, 0)))
 	    {
 	      if (!NILP (full))
 		{
-		  int afterdirindex = dirnamelen;
-		  int total = len + dirnamelen;
-		  int needsep = 0;
-
-		  /* Decide whether we need to add a directory separator.  */
+		  int index = XSTRING (dirname)->size;
+		  int total = len + index;
 #ifndef VMS
-		  if (dirnamelen == 0
-		      || !IS_ANY_SEP (XSTRING (directory)->data[dirnamelen - 1]))
-		    needsep = 1;
+		  if (length == 0
+		      || XSTRING (dirname)->data[length - 1] != '/')
+		    total++;
 #endif /* VMS */
 
-		  name = make_uninit_string (total + needsep);
-		  bcopy (XSTRING (directory)->data, XSTRING (name)->data,
-			 dirnamelen);
-		  if (needsep)
-		    XSTRING (name)->data[afterdirindex++] = DIRECTORY_SEP;
-		  bcopy (dp->d_name,
-			 XSTRING (name)->data + afterdirindex, len);
+		  name = make_uninit_string (total);
+		  bcopy (XSTRING (dirname)->data, XSTRING (name)->data,
+			 index);
+#ifndef VMS
+		  if (length == 0
+		      || XSTRING (dirname)->data[length - 1] != '/')
+		    XSTRING (name)->data[index++] = '/';
+#endif /* VMS */
+		  bcopy (dp->d_name, XSTRING (name)->data + index, len);
 		}
 	      else
 		name = make_string (dp->d_name, len);
@@ -230,53 +226,48 @@ Lisp_Object file_name_completion ();
 
 DEFUN ("file-name-completion", Ffile_name_completion, Sfile_name_completion,
   2, 2, 0,
-  "Complete file name FILE in directory DIRECTORY.\n\
+  "Complete file name FILE in directory DIR.\n\
 Returns the longest string\n\
-common to all file names in DIRECTORY that start with FILE.\n\
+common to all filenames in DIR that start with FILE.\n\
 If there is only one and FILE matches it exactly, returns t.\n\
 Returns nil if DIR contains no name starting with FILE.")
-  (file, directory)
-     Lisp_Object file, directory;
+  (file, dirname)
+     Lisp_Object file, dirname;
 {
   Lisp_Object handler;
-
-  /* If the directory name has special constructs in it,
-     call the corresponding file handler.  */
-  handler = Ffind_file_name_handler (directory, Qfile_name_completion);
-  if (!NILP (handler))
-    return call3 (handler, Qfile_name_completion, file, directory);
+  /* Don't waste time trying to complete a null string.
+     Besides, this case happens when user is being asked for
+     a directory name and has supplied one ending in a /.
+     We would not want to add anything in that case
+     even if there are some unique characters in that directory.  */
+  if (XTYPE (file) == Lisp_String && XSTRING (file)->size == 0)
+    return file;
 
   /* If the file name has special constructs in it,
      call the corresponding file handler.  */
-  handler = Ffind_file_name_handler (file, Qfile_name_completion);
+  handler = Ffind_file_name_handler (dirname, Qfile_name_completion);
   if (!NILP (handler))
-    return call3 (handler, Qfile_name_completion, file, directory);
+    return call3 (handler, Qfile_name_completion, file, dirname);
 
-  return file_name_completion (file, directory, 0, 0);
+  return file_name_completion (file, dirname, 0, 0);
 }
 
 DEFUN ("file-name-all-completions", Ffile_name_all_completions,
   Sfile_name_all_completions, 2, 2, 0,
-  "Return a list of all completions of file name FILE in directory DIRECTORY.\n\
-These are all file names in directory DIRECTORY which begin with FILE.")
-  (file, directory)
-     Lisp_Object file, directory;
+  "Return a list of all completions of file name FILE in directory DIR.\n\
+These are all file names in directory DIR which begin with FILE.")
+  (file, dirname)
+     Lisp_Object file, dirname;
 {
   Lisp_Object handler;
 
-  /* If the directory name has special constructs in it,
-     call the corresponding file handler.  */
-  handler = Ffind_file_name_handler (directory, Qfile_name_all_completions);
-  if (!NILP (handler))
-    return call3 (handler, Qfile_name_all_completions, file, directory);
-
   /* If the file name has special constructs in it,
      call the corresponding file handler.  */
-  handler = Ffind_file_name_handler (file, Qfile_name_all_completions);
+  handler = Ffind_file_name_handler (dirname, Qfile_name_all_completions);
   if (!NILP (handler))
-    return call3 (handler, Qfile_name_all_completions, file, directory);
+    return call3 (handler, Qfile_name_all_completions, file, dirname);
 
-  return file_name_completion (file, directory, 1, 0);
+  return file_name_completion (file, dirname, 1, 0);
 }
 
 Lisp_Object
@@ -297,19 +288,6 @@ file_name_completion (file, dirname, all_flag, ver_flag)
   int count = specpdl_ptr - specpdl;
   struct gcpro gcpro1, gcpro2, gcpro3;
 
-#ifdef MSDOS
-#if __DJGPP__ > 1
-  /* Some fields of struct stat are *very* expensive to compute on MS-DOS,
-     but aren't required here.  Avoid computing the following fields:
-     st_inode, st_size and st_nlink for directories, and the execute bits
-     in st_mode for non-directory files with non-standard extensions.  */
-
-  unsigned short save_djstat_flags = _djstat_flags;
-
-  _djstat_flags = _STAT_INODE | _STAT_EXEC_MAGIC | _STAT_DIRSIZE;
-#endif
-#endif
-
 #ifdef VMS
   extern DIRENTRY * readdirver ();
 
@@ -375,17 +353,7 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 
           directoryp = ((st.st_mode & S_IFMT) == S_IFDIR);
 	  tem = Qnil;
-          if (directoryp)
-	    {
-#ifndef TRIVIAL_DIRECTORY_ENTRY
-#define TRIVIAL_DIRECTORY_ENTRY(n) (!strcmp (n, ".") || !strcmp (n, ".."))
-#endif
-	      /* "." and ".." are never interesting as completions, but are
-		 actually in the way in a directory contains only one file.  */
-	      if (!passcount && TRIVIAL_DIRECTORY_ENTRY (dp->d_name))
-		continue;
-	    }
-	  else
+          if (!directoryp)
             {
 	      /* Compare extensions-to-be-ignored against end of this file name */
 	      /* if name is not an exact match against specified string */
@@ -395,7 +363,7 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 		     CONSP (tem); tem = XCONS (tem)->cdr)
 		  {
 		    elt = XCONS (tem)->car;
-		    if (!STRINGP (elt)) continue;
+		    if (XTYPE (elt) != Lisp_String) continue;
 		    skip = len - XSTRING (elt)->size;
 		    if (skip < 0) continue;
 
@@ -416,7 +384,7 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 	    {
 	      Lisp_Object regexps;
 	      Lisp_Object zero;
-	      XSETFASTINT (zero, 0);
+	      XFASTINT (zero) = 0;
 
 	      /* Ignore this element if it fails to match all the regexps.  */
 	      for (regexps = Vcompletion_regexp_list; CONSP (regexps);
@@ -468,8 +436,6 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 		     use it as the best match rather than one that is not
 		     an exact match.  This way, we get the case pattern
 		     of the actual match.  */
-		  /* This tests that the current file is an exact match
-		     but BESTMATCH is not (it is too long).  */
 		  if ((matchsize == len
 		       && matchsize + !!directoryp 
 			  < XSTRING (bestmatch)->size)
@@ -477,15 +443,13 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 		      /* If there is no exact match ignoring case,
 			 prefer a match that does not change the case
 			 of the input.  */
-		      /* If there is more than one exact match aside from
-			 case, and one of them is exact including case,
-			 prefer that one.  */
-		      /* This == checks that, of current file and BESTMATCH,
-			 either both or neither are exact.  */
 		      (((matchsize == len)
 			==
 			(matchsize + !!directoryp 
 			 == XSTRING (bestmatch)->size))
+		       /* If there is more than one exact match aside from
+			  case, and one of them is exact including case,
+			  prefer that one.  */
 		       && !bcmp (p2, XSTRING (file)->data, XSTRING (file)->size)
 		       && bcmp (p1, XSTRING (file)->data, XSTRING (file)->size)))
 		    {
@@ -500,7 +464,7 @@ file_name_completion (file, dirname, all_flag, ver_flag)
 	      if (directoryp
 		  && compare == matchsize
 		  && bestmatchsize > matchsize
-		  && IS_ANY_SEP (p1[matchsize]))
+		  && p1[matchsize] == '/')
 		matchsize++;
 	      bestmatchsize = matchsize;
 	    }
@@ -511,12 +475,6 @@ file_name_completion (file, dirname, all_flag, ver_flag)
   UNGCPRO;
   bestmatch = unbind_to (count, bestmatch);
 
-#ifdef MSDOS
-#if __DJGPP__ > 1
-  _djstat_flags = save_djstat_flags;
-#endif
-#endif
-
   if (all_flag || NILP (bestmatch))
     return bestmatch;
   if (matchcount == 1 && bestmatchsize == XSTRING (file)->size)
@@ -540,8 +498,8 @@ file_name_completion_stat (dirname, dp, st_addr)
 
   bcopy (XSTRING (dirname)->data, fullname, pos);
 #ifndef VMS
-  if (!IS_DIRECTORY_SEP (fullname[pos - 1]))
-    fullname[pos++] = DIRECTORY_SEP;
+  if (fullname[pos - 1] != '/')
+    fullname[pos++] = '/';
 #endif
 
   bcopy (dp->d_name, fullname + pos, len);
@@ -563,11 +521,11 @@ file_name_completion_stat (dirname, dp, st_addr)
 
 DEFUN ("file-name-all-versions", Ffile_name_all_versions,
   Sfile_name_all_versions, 2, 2, 0,
-  "Return a list of all versions of file name FILE in directory DIRECTORY.")
-  (file, directory)
-     Lisp_Object file, directory;
+  "Return a list of all versions of file name FILE in directory DIR.")
+  (file, dirname)
+     Lisp_Object file, dirname;
 {
-  return file_name_completion (file, directory, 1, 1);
+  return file_name_completion (file, dirname, 1, 1);
 }
 
 DEFUN ("file-version-limit", Ffile_version_limit, Sfile_version_limit, 1, 1, 0,
@@ -648,6 +606,22 @@ If file does not exist, returns nil.")
   if (lstat (XSTRING (filename)->data, &s) < 0)
     return Qnil;
 
+#ifdef MSDOS
+  {
+    char *tmpnam = XSTRING (Ffile_name_nondirectory (filename))->data;
+    int l = strlen (tmpnam);
+
+    if (l >= 5 
+	&& S_ISREG (s.st_mode)
+	&& (stricmp (&tmpnam[l - 4], ".com") == 0
+	    || stricmp (&tmpnam[l - 4], ".exe") == 0
+	    || stricmp (&tmpnam[l - 4], ".bat") == 0))
+      {
+	s.st_mode |= S_IEXEC;
+      }
+  }
+#endif /* MSDOS */
+
   switch (s.st_mode & S_IFMT)
     {
     default:
