@@ -15,11 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
-
-#ifndef _DISPEXTERN_H_
-#define _DISPEXTERN_H_
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* 89.2.9   modified for Nemacs Ver.3.0 by K.Handa */
 /* 89.11.24 modified for attribute of characters by K.Handa */
@@ -60,30 +56,30 @@ extern int display_completed;
 
 #ifdef HAVE_X_WINDOWS
 #include <X11/Xlib.h>
-#endif
+#endif /* HAVE_X_WINDOWS */
 
-#ifdef MSDOS
-#include "msdos.h"
-#endif
-
-#ifdef HAVE_NTGUI
-#include "win32.h"
-#endif
-
-#ifdef HAVE_FACES
+#if defined (HAVE_X_WINDOWS) || defined (TERMINAL_FACE)
 struct face
   {
 #ifdef HAVE_X_WINDOWS
     /* If this is non-zero, it is a GC we can use without modification
        to represent this face.  */
     GC gc;
+
 #endif /* HAVE_X_WINDOWS */
 
+    /* If we have ever called get_cached_face on this face structure,
+       here is the index in face_vector of the face it returned.  It
+       might not be valid any more, but it's a good place to start
+       looking; get_cached_face tries to use this to avoid searching
+       all of face_vector.  */
+    int cached_index;
+  
     /* Pixel value for foreground color.  */
-    EMACS_UINT foreground;
+    Lisp_Object_Int foreground;
   
     /* Pixel value for background color.  */
-    EMACS_UINT background;
+    Lisp_Object_Int background;
   
 #ifdef HAVE_X_WINDOWS
     /* Font used for this face.  */
@@ -144,7 +140,7 @@ typedef struct face *FACE;
 #define FACE_FOREGROUND(f) ((f)->foreground)
 #define FACE_BACKGROUND(f) ((f)->background)
 
-#else /* not HAVE_FACES */
+#else  /* Not X nor TERMINAL_FACE */
 
 typedef int FACE;
 
@@ -155,9 +151,7 @@ typedef int FACE;
 
 #define FACE_HIGHLIGHT(f) ((f) & 0x1)
 #define FACE_UNDERLINE(f) ((f) & 0x2)
-
-#endif /* not HAVE_FACES */
-
+#endif /* Not X nor TERMINAL_FACE */
 
 /* This structure is used for the actual display of text on a frame.
 
@@ -198,9 +192,9 @@ struct frame_glyphs
     /* Buffer offset of this line's first char.
        This is not really implemented, and cannot be,
        and should be deleted.  */
-    EMACS_INT *bufp;
+    Lisp_Object_Int *bufp;
 
-#ifdef HAVE_WINDOW_SYSTEM
+#ifdef HAVE_X_WINDOWS
     /* Pixel position of top left corner of line.  */
     short *top_left_x;
     short *top_left_y;
@@ -213,7 +207,7 @@ struct frame_glyphs
 
     /* Largest font ascent on this line.  */
     short *max_ascent;
-#endif	/* HAVE_WINDOW_SYSTEM */
+#endif	/* HAVE_X_WINDOWS */
 
     /* Mapping of coordinate pairs to buffer positions.
        This field holds a vector indexed by row number.
@@ -248,8 +242,10 @@ struct frame_glyphs
 extern void get_display_line ();
 extern Lisp_Object sit_for ();
 
+#define MESSAGE_BUFFER_SIZE(frame) (FRAME_WIDTH (frame) * 3)
+#define MSG_BUF_SIZE (MESSAGE_BUFFER_SIZE (selected_frame))
+
 /* 93.4.13 K.Handa -- Nonzero means highlight anti-direction characters. */
 extern int highlight_reverse_direction;
 /* 93.5.22 Y.Niibe -- Nonzero means display double cursor */
 extern int r2l_double_cursor;
-#endif /* not _DISPEXTERN_H_ */
