@@ -1,5 +1,5 @@
 /* Fully extensible Emacs, running on Unix, intended for GNU.
-   Copyright (C) 1985, 86, 87, 93, 94, 95 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* 87.6.9   modified by K.Handa */
 /* 88.1.10  modified for Nemacs Ver.2.0 by K.Handa */
@@ -73,6 +72,12 @@ Boston, MA 02111-1307, USA.  */
 #include <sys/ioctl.h>
 #endif
 
+#ifdef APOLLO
+#ifndef APOLLO_SR10
+#include <default_acl.h>
+#endif
+#endif
+
 /* 92.3.31 by K.Handa */
 #ifdef HAVE_SUN_CONSOLE
 #include <pixrect/pixrect_hs.h>
@@ -96,7 +101,6 @@ Boston, MA 02111-1307, USA.  */
 #include "intervals.h"
 
 #include "systty.h"
-#include "blockinput.h"
 #include "syssignal.h"
 #include "process.h"
 
@@ -105,7 +109,6 @@ Boston, MA 02111-1307, USA.  */
 #endif
 
 extern void malloc_warning ();
-extern void set_time_zone_rule ();
 extern char *index ();
 extern char *strerror ();
 
@@ -136,11 +139,7 @@ Lisp_Object Vsystem_type;
 
 /* Variable whose value is string giving configuration built for.  */
 Lisp_Object Vsystem_configuration;
-
-/* Variable whose value is string giving configuration options,
-   for use when reporting bugs.  */
-Lisp_Object Vsystem_configuration_options;
-
+  
 /* If non-zero, emacs should not attempt to use an window-specific code,
    but instead should use the virtual terminal under which it was started */
 /* 92.3.31 by K.Handa - If negative, Mule use Sun console feature. */
@@ -151,10 +150,6 @@ int inhibit_window_system;
    priority; Those functions have their own extern declaration.  */
 int emacs_priority;
 
-/* If non-zero a filter or a sentinel is running.  Tested to save the match
-   data on the first attempt to change it inside asynchronous code. */
-int running_asynch_code;
-
 #ifdef BSD_PGRPS
 /* See sysdep.c.  */
 extern int inherited_pgroup;
@@ -170,11 +165,9 @@ int display_arg;
    Tells GC how to save a copy of the stack.  */
 char *stack_bottom;
 
-#ifdef HAVE_WINDOW_SYSTEM
+#ifdef HAVE_X_WINDOWS
 extern Lisp_Object Vwindow_system;
-#endif /* HAVE_WINDOW_SYSTEM */
-
-extern Lisp_Object Vauto_save_list_file_name;
+#endif /* HAVE_X_WINDOWS */
 
 #ifdef USG_SHARED_LIBRARIES
 /* If nonzero, this is the place to put the end of the writable segment
@@ -196,8 +189,6 @@ int noninteractive1;
 /* Save argv and argc.  */
 char **initial_argv;
 int initial_argc;
-
-static void sort_args ();
 
 /* Signal code for the fatal signal that was received */
 int fatal_error_code;
@@ -213,8 +204,6 @@ fatal_error_signal (sig)
   fatal_error_code = sig;
   signal (sig, SIG_DFL);
 
-  TOTALLY_UNBLOCK_INPUT;
-
   /* If fatal error occurs in code below, avoid infinite recursion.  */
   if (! fatal_error_in_progress)
     {
@@ -280,11 +269,6 @@ init_cmdargs (argc, argv, skip_args)
 	Vinvocation_directory = Ffile_name_directory (found);
     }
 
-  if (!NILP (Vinvocation_directory)
-      && NILP (Ffile_name_absolute_p (Vinvocation_directory)))
-    /* Emacs was started with relative path, like ./emacs  */
-    Vinvocation_directory = Fexpand_file_name (Vinvocation_directory, Qnil);
-
   Vinstallation_directory = Qnil;
 
   if (!NILP (Vinvocation_directory))
@@ -371,12 +355,11 @@ DEFUN ("invocation-directory", Finvocation_directory, Sinvocation_directory,
 
 #ifdef VMS
 #ifdef LINK_CRTL_SHARE
-#ifdef SHARABLE_LIB_BUG
+#ifdef SHAREABLE_LIB_BUG
 extern noshare char **environ;
-#endif /* SHARABLE_LIB_BUG */
+#endif /* SHAREABLE_LIB_BUG */
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
-
 /* 92.11.4 by M.Higashida */
 #if defined(MSDOS) && defined(EMX)
 extern char **environ;
@@ -397,12 +380,6 @@ asm ("	.stabs	\"___CTOR_LIST__\", 23, 0, 0, L_setbinmode");
 #endif
 /* end of patch */
 
-#ifdef HAVE_TZSET
-/* A valid but unlikely value for the TZ environment value.
-   It is OK (though a bit slower) if the user actually chooses this value.  */
-static char dump_tz[] = "UtC0";
-#endif
-
 #ifndef ORDINARY_LINK
 /* We don't include crtbegin.o and crtend.o in the link,
    so these functions and variables might be missed.
@@ -427,75 +404,6 @@ __main ()
 #endif /* __GNUC__ */
 #endif /* ORDINARY_LINK */
 
-/* Test whether the next argument in ARGV matches SSTR or a prefix of
-   LSTR (at least MINLEN characters).  If so, then if VALPTR is non-null
-   (the argument is supposed to have a value) store in *VALPTR either
-   the next argument or the portion of this one after the equal sign.
-   ARGV is read starting at position *SKIPPTR; this index is advanced
-   by the number of arguments used.
-
-   Too bad we can't just use getopt for all of this, but we don't have
-   enough information to do it right.  */
-
-static int
-argmatch (argv, argc, sstr, lstr, minlen, valptr, skipptr)
-     char **argv;
-     int argc;
-     char *sstr;
-     char *lstr;
-     int minlen;
-     char **valptr;
-     int *skipptr;
-{
-  char *p;
-  int arglen;
-  char *arg;
-
-  /* Don't access argv[argc]; give up in advance.  */
-  if (argc <= *skipptr + 1)
-    return 0;
-
-  arg = argv[*skipptr+1];
-  if (arg == NULL)
-    return 0;
-  if (strcmp (arg, sstr) == 0)
-    {
-      if (valptr != NULL)
-	{
-	  *valptr = argv[*skipptr+2];
-	  *skipptr += 2;
-	}
-      else
-	*skipptr += 1;
-      return 1;
-    }
-  arglen = (valptr != NULL && (p = index (arg, '=')) != NULL
-	    ? p - arg : strlen (arg));
-  if (lstr == 0 || arglen < minlen || strncmp (arg, lstr, arglen) != 0)
-    return 0;
-  else if (valptr == NULL)
-    {
-      *skipptr += 1;
-      return 1;
-    }
-  else if (p != NULL)
-    {
-      *valptr = p+1;
-      *skipptr += 1;
-      return 1;
-    }
-  else if (argv[*skipptr+2] != NULL)
-    {
-      *valptr = argv[*skipptr+2];
-      *skipptr += 2;
-      return 1;
-    }
-  else
-    {
-      return 0;
-    }
-}
-
 #ifdef NeXT
 #include <objc/zone.h>
 #include <mach/mach.h>
@@ -513,31 +421,9 @@ main (argc, argv, envp)
   extern int errno;
   extern sys_nerr;
 
-#ifdef LINUX_SBRK_BUG
-  __sbrk (1);
-#endif
-
-  sort_args (argc, argv);
-
-  if (argmatch (argv, argc, "-version", "--version", 3, NULL, &skip_args))
-    {
-      Lisp_Object tem;
-      tem = Fsymbol_value (intern ("emacs-version"));
-      if (!STRINGP (tem))
-	{
-	  fprintf (stderr, "Invalid value of `emacs-version'\n");
-	  exit (1);
-	}
-      else
-	{
-	  printf ("GNU Emacs %s\n", XSTRING (tem)->data);
-	  exit (0);
-	}
-    }
-
 /* Map in shared memory, if we are using that.  */
 #ifdef HAVE_SHM
-  if (argmatch (argv, argc, "-nl", "--no-shared-memory", 6, NULL, &skip_args))
+  if (argc > 1 && !strcmp (argv[1], "-nl"))
     {
       map_in_data (0);
       /* The shared memory was just restored, which clobbered this.  */
@@ -552,33 +438,46 @@ main (argc, argv, envp)
 #endif
 
 #ifdef NeXT
-  {
-    extern int malloc_cookie;
-    /* This helps out unexnext.c.  */
-    if (initialized)
-      {
-	if (malloc_jumpstart (malloc_cookie) != 0)
-	  printf ("malloc jumpstart failed!\n");
-      }
-    else
-      {
-	emacszone = NXCreateZone(ZONESIZE*vm_page_size,vm_page_size,1);
-	if(emacszone == NX_NOZONE)
-	  fatal("can't create emacs zone.\n");
-      }
-  }
+  extern int malloc_cookie;
+
+  /* This helps out unexnext.c.  */
+  if (initialized)
+    {
+      if (malloc_jumpstart (malloc_cookie) != 0)
+	fatal("malloc jumpstart failed!\n");
+    }
+  else
+    {
+      emacszone = NXCreateZone(ZONESIZE*vm_page_size,vm_page_size,1);
+      if(emacszone == NX_NOZONE)
+	fatal("can't create emacs zone.\n");
+    }
 #endif /* NeXT */
 
+#ifdef HAVE_X_WINDOWS
+  /* Stupid kludge to catch command-line display spec.  We can't
+     handle this argument entirely in window system dependent code
+     because we don't even know which window system dependent code
+     to run until we've recognized this argument.  */
+  {
+    int i;
+
+    for (i = 1; (i < argc && ! display_arg); i++)
+      if (!strcmp (argv[i], "-d") || !strcmp (argv[i], "-display"))
+	display_arg = 1;
+  }
+#endif
+
 #ifdef VMS
   /* If -map specified, map the data file in */
-  {
-    char *file;
-    if (argmatch (argv, argc, "-map", "--map-data", 3, &mapin_file, &skip_args))
-      mapin_data (file);
-  }
+  if (argc > 2 && ! strcmp (argv[1], "-map"))
+    {
+      skip_args = 2;
+      mapin_data (argv[2]);
+    }
 
 #ifdef LINK_CRTL_SHARE
-#ifdef SHARABLE_LIB_BUG
+#ifdef SHAREABLE_LIB_BUG
   /* Bletcherous shared libraries! */
   if (!stdin)
     stdin = fdopen (0, "r");
@@ -588,7 +487,7 @@ main (argc, argv, envp)
     stderr = fdopen (2, "w");
   if (!environ)
     environ = envp;
-#endif /* SHARABLE_LIB_BUG */
+#endif /* SHAREABLE_LIB_BUG */
 #endif /* LINK_CRTL_SHARE */
 #endif /* VMS */
 /* 92.11.4, 93.2.17 by M.Higashida */
@@ -610,11 +509,20 @@ main (argc, argv, envp)
 
 #ifdef USG_SHARED_LIBRARIES
   if (bss_end)
-    brk ((void *)bss_end);
+    brk (bss_end);
 #endif
 
   clearerr (stdin);
 
+#ifdef APOLLO
+#ifndef APOLLO_SR10
+  /* If USE_DOMAIN_ACLS environment variable exists,
+     use ACLs rather than UNIX modes. */
+  if (egetenv ("USE_DOMAIN_ACLS"))
+    default_acl (USE_DEFACL);
+#endif
+#endif /* APOLLO */
+
 #ifndef SYSTEM_MALLOC
   if (! initialized)
     {
@@ -631,20 +539,9 @@ main (argc, argv, envp)
   /* We do all file input/output as binary files.  When we need to translate
      newlines, we do that manually.  */
   _fmode = O_BINARY;
-
-#if __DJGPP__ >= 2
-  if (!isatty (fileno (stdin)))
-    setmode (fileno (stdin), O_BINARY);
-  if (!isatty (fileno (stdout)))
-    {
-      fflush (stdout);
-      setmode (fileno (stdout), O_BINARY);
-    }
-#else  /* not __DJGPP__ >= 2 */
   (stdin)->_flag &= ~_IOTEXT;
   (stdout)->_flag &= ~_IOTEXT;
   (stderr)->_flag &= ~_IOTEXT;
-#endif /* not __DJGPP__ >= 2 */
 #endif /* MSDOS */
 
 #ifdef SET_EMACS_PRIORITY
@@ -660,93 +557,46 @@ main (argc, argv, envp)
   inhibit_window_system = 0;
 
   /* Handle the -t switch, which specifies filename to use as terminal */
-  {
-    char *term;
-    if (argmatch (argv, argc, "-t", "--terminal", 4, &term, &skip_args))
-      {
-	int result;
-	close (0);
-	close (1);
-	result = open (term, O_RDWR, 2 );
-	if (result < 0)
-	  {
-	    char *errstring = strerror (errno);
-	    fprintf (stderr, "emacs: %s: %s\n", term, errstring);
-	    exit (1);
-	  }
-	dup (0);
-	if (! isatty (0))
-	  {
-	    fprintf (stderr, "emacs: %s: not a tty\n", term);
-	    exit (1);
-	  }
-	fprintf (stderr, "Using %s\n", term);
-#ifdef HAVE_WINDOW_SYSTEM
-	inhibit_window_system = 1; /* -t => -nw */
-#endif
-      }
-  }
-  if (argmatch (argv, argc, "-nw", "--no-windows", 6, NULL, &skip_args))
-    inhibit_window_system = 1;
-
-  /* Handle the -batch switch, which means don't do interactive display.  */
-  noninteractive = 0;
-  if (argmatch (argv, argc, "-batch", "--batch", 5, NULL, &skip_args))
-    noninteractive = 1;
-
-  /* Handle the --help option, which gives a usage message..  */
-  if (argmatch (argv, argc, "-help", "--help", 3, NULL, &skip_args))
+  if (skip_args + 2 < argc && !strcmp (argv[skip_args + 1], "-t"))
     {
-      printf ("\
-Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
-      [-q] [--no-init-file]  [-u user] [--user user]  [--debug-init]\n\
-      [--version] [--no-site-file]\n\
-      [-f func] [--funcall func]  [-l file] [--load file]  [--insert file]\n\
-      [+linenum] file-to-visit  [--kill]\n", argv[0]);
-      exit (0);
+      int result;
+      skip_args += 2;
+      close (0);
+      close (1);
+      result = open (argv[skip_args], O_RDWR, 2 );
+      if (result < 0)
+	{
+	  char *errstring = strerror (errno);
+	  fprintf (stderr, "emacs: %s: %s\n", argv[skip_args], errstring);
+	  exit (1);
+	}
+      dup (0);
+      if (! isatty (0))
+	{
+	  fprintf (stderr, "emacs: %s: not a tty\n", argv[skip_args]);
+	  exit (1);
+	}
+      fprintf (stderr, "Using %s\n", argv[skip_args]);
+/* 92.3.31 by K.Handa, 92.10.21 by M.Higashida */
+#if defined (HAVE_X_WINDOWS) || defined (HAVE_SUN_CONSOLE) || (defined(MSDOS) && defined(HAVE_VGA_ADAPTER)) || defined (WIN32)
+      inhibit_window_system = 1;	/* -t => -nw */
+#endif
     }
 
-#ifdef HAVE_X_WINDOWS
-  /* Stupid kludge to catch command-line display spec.  We can't
-     handle this argument entirely in window system dependent code
-     because we don't even know which window system dependent code
-     to run until we've recognized this argument.  */
-  {
-    char *displayname = 0;
-    int i;
-    int count_before = skip_args;
+  if (skip_args + 1 < argc
+      && (!strcmp (argv[skip_args + 1], "-nw")))
+    {
+      skip_args += 1;
+      inhibit_window_system = 1;
+    }
 
-    if (argmatch (argv, argc, "-d", "--display", 3, &displayname, &skip_args))
-      display_arg = 1;
-    else if (argmatch (argv, argc, "-display", 0, 3, &displayname, &skip_args))
-      display_arg = 1;
-
-    /* If we have the form --display=NAME,
-       convert it into  -d name.
-       This requires inserting a new element into argv.  */
-    if (displayname != 0 && skip_args - count_before == 1)
-      {
-	char **new = (char **) xmalloc (sizeof (char *) * (argc + 2));
-	int j;
-
-	for (j = 0; j < count_before + 1; j++)
-	  new[j] = argv[j];
-	new[count_before + 1] = "-d";
-	new[count_before + 2] = displayname;
-	for (j = count_before + 2; j <argc; j++)
-	  new[j + 1] = argv[j];
-	argv = new;
-	argc++;
-      }
-    /* Change --display to -d, when its arg is separate.  */
-    else if (displayname != 0 && skip_args > count_before
-	     && argv[count_before + 1][1] == '-')
-      argv[count_before + 1] = "-d";
-
-    /* Don't actually discard this arg.  */
-    skip_args = count_before;
-  }
-#endif
+/* Handle the -batch switch, which means don't do interactive display.  */
+  noninteractive = 0;
+  if (skip_args + 1 < argc && !strcmp (argv[skip_args + 1], "-batch"))
+    {
+      skip_args += 1;
+      noninteractive = 1;
+    }
 
   if (! noninteractive)
     {
@@ -767,22 +617,6 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
   init_signals ();
 #endif
 
-  /* Don't catch SIGHUP if dumping.  */
-  if (1
-#ifndef CANNOT_DUMP
-      && initialized
-#endif
-      )
-    {
-      sigblock (sigmask (SIGHUP));
-      /* In --batch mode, don't catch SIGHUP if already ignored.
-	 That makes nohup work.  */
-      if (! noninteractive
-	  || signal (SIGHUP, SIG_IGN) != SIG_IGN)
-	signal (SIGHUP, fatal_error_signal);
-      sigunblock (sigmask (SIGHUP));
-    }
-
   if (
 #ifndef CANNOT_DUMP
       ! noninteractive || initialized
@@ -791,33 +625,13 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
 #endif
       )
     {
-      /* Don't catch these signals in batch mode if dumping.
+      /* Don't catch these signals in batch mode if not initialized.
 	 On some machines, this sets static data that would make
 	 signal fail to work right when the dumped Emacs is run.  */
+      signal (SIGHUP, fatal_error_signal);
       signal (SIGQUIT, fatal_error_signal);
       signal (SIGILL, fatal_error_signal);
       signal (SIGTRAP, fatal_error_signal);
-#ifdef SIGABRT
-      signal (SIGABRT, fatal_error_signal);
-#endif
-#ifdef SIGHWE
-      signal (SIGHWE, fatal_error_signal);
-#endif
-#ifdef SIGPRE
-      signal (SIGPRE, fatal_error_signal);
-#endif
-#ifdef SIGORE
-      signal (SIGORE, fatal_error_signal);
-#endif
-#ifdef SIGUME
-      signal (SIGUME, fatal_error_signal);
-#endif
-#ifdef SIGDLK
-      signal (SIGDLK, fatal_error_signal);
-#endif
-#ifdef SIGCPULIM
-      signal (SIGCPULIM, fatal_error_signal);
-#endif
 #ifdef SIGIOT
       /* This is missing on some systems - OS/2, for example.  */
       signal (SIGIOT, fatal_error_signal);
@@ -891,22 +705,12 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
 #ifdef MULE			/* 94.5.25 by K.Handa */
   init_fontset ();
 #endif  /* MULE */
-  running_asynch_code = 0;
 
 #ifdef MSDOS
   /* Call early 'cause init_environment needs it.  */
   init_dosfns ();
   /* Set defaults for several environment variables.  */
-  if (initialized)
-    init_environment (argc, argv, skip_args);
-  else
-    tzset ();
-#endif /* MSDOS */
-
-#ifdef WINDOWSNT
-  /* Initialize environment from registry settings.  */
-  init_environment ();
-  init_ntproc ();	/* must precede init_editfns */
+  if (initialized) init_environment (argc, argv, skip_args);
 #endif
 
   /* egetenv is a pretty low-level facility, which may get called in
@@ -1016,9 +820,6 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
 #ifdef VMS
       syms_of_vmsproc ();
 #endif /* VMS */
-#ifdef WINDOWSNT
-      syms_of_ntproc ();
-#endif /* WINDOWSNT */
       syms_of_window ();
       syms_of_xdisp ();
 #ifdef HAVE_X_WINDOWS
@@ -1027,23 +828,14 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
 #ifdef HAVE_X11
       syms_of_xselect ();
 #endif
+#ifdef HAVE_X_MENU
+      syms_of_xmenu ();
+#endif /* HAVE_X_MENU */
 #endif /* HAVE_X_WINDOWS */
 
-#ifdef HAVE_FACES
+#if defined (HAVE_X_WINDOWS) || defined (TERMINAL_FACE)
       syms_of_xfaces ();
-#endif
-
-#ifndef HAVE_NTGUI
-      syms_of_xmenu ();
-#endif
-
-#ifdef HAVE_NTGUI
-      syms_of_win32term ();
-      syms_of_win32fns ();
-      syms_of_win32faces ();
-      syms_of_win32select ();
-      syms_of_win32menu ();
-#endif /* HAVE_NTGUI */
+#endif /* HAVE_X_WINDOWS or TERMINAL_FACE */
 
 /* 87.6.8, 91.11.2, 92.3.31, 92.7.31  by K.Handa, 92.10.17, 93.2.17 by M.Higashida */
 #ifdef HAVE_SUN_CONSOLE
@@ -1091,257 +883,34 @@ Usage: %s [-t term] [--terminal term]  [-nw] [--no-windows]  [--batch]\n\
 
   if (!initialized)
     {
-      char *file;
       /* Handle -l loadup-and-dump, args passed by Makefile. */
-      if (argmatch (argv, argc, "-l", "--load", 3, &file, &skip_args))
+      if (argc > 2 + skip_args && !strcmp (argv[1 + skip_args], "-l"))
 	Vtop_level = Fcons (intern ("load"),
-			    Fcons (build_string (file), Qnil));
+			    Fcons (build_string (argv[2 + skip_args]), Qnil));
 #ifdef CANNOT_DUMP
       /* Unless next switch is -nl, load "loadup.el" first thing.  */
-      if (!argmatch (argv, argc, "-nl", "--no-loadup", 6, NULL, &skip_args))
+      if (!(argc > 1 + skip_args && !strcmp (argv[1 + skip_args], "-nl")))
 	Vtop_level = Fcons (intern ("load"),
 			    Fcons (build_string ("loadup.el"), Qnil));
 #endif /* CANNOT_DUMP */
     }
 
-  if (initialized)
-    {
-      /* Erase any pre-dump messages in the message log, to avoid confusion */
-      Lisp_Object old_log_max;
-      old_log_max = Vmessage_log_max;
-      XSETFASTINT (Vmessage_log_max, 0);
-      message_dolog ("", 0, 1);
-      Vmessage_log_max = old_log_max;
-
-#ifdef HAVE_TZSET
-      {
-	/* If the execution TZ happens to be the same as the dump TZ,
-	   change it to some other value and then change it back,
-	   to force the underlying implementation to reload the TZ info.
-	   This is needed on implementations that load TZ info from files,
-	   since the TZ file contents may differ between dump and execution.  */
-	char *tz = getenv ("TZ");
-	if (tz && !strcmp (tz, dump_tz))
-	  {
-	    ++*tz;
-	    tzset ();
-	    --*tz;
-	  }
-      }
-#endif
-    }
-
   initialized = 1;
 
-#ifdef LOCALTIME_CACHE
-  /* Some versions of localtime have a bug.  They cache the value of the time
+#if defined (sun) || defined (LOCALTIME_CACHE)
+  /* sun's localtime has a bug.  it caches the value of the time
      zone rather than looking it up every time.  Since localtime() is
      called to bolt the undumping time into the undumped emacs, this
      results in localtime ignoring the TZ environment variable.
      This flushes the new TZ value into localtime. */
   tzset ();
-#endif /* defined (LOCALTIME_CACHE) */
+#endif /* defined (sun) || defined (LOCALTIME_CACHE) */
 
   /* Enter editor command loop.  This never returns.  */
   Frecursive_edit ();
   /* NOTREACHED */
 }
 
-/* Sort the args so we can find the most important ones
-   at the beginning of argv.  */
-
-/* First, here's a table of all the standard options.  */
-
-struct standard_args
-{
-  char *name;
-  char *longname;
-  int priority;
-  int nargs;
-};
-
-struct standard_args standard_args[] =
-{
-  { "-version", "--version", 110, 0 },
-  { "-help", "--help", 110, 0 },
-  { "-nl", "--no-shared-memory", 100, 0 },
-#ifdef VMS
-  { "-map", "--map-data", 100, 0 },
-#endif
-  { "-t", "--terminal", 90, 1 },
-  { "-d", "--display", 80, 1 },
-  { "-display", 0, 80, 1 },
-  { "-nw", "--no-windows", 70, 0 },
-  { "-batch", "--batch", 60, 0 },
-  { "-q", "--no-init-file", 50, 0 },
-  { "-no-init-file", 0, 50, 0 },
-  { "-no-site-file", "--no-site-file", 40, 0 },
-  { "-u", "--user", 30, 1 },
-  { "-user", 0, 30, 1 },
-  { "-debug-init", "--debug-init", 20, 0 },
-  { "-i", "--icon-type", 15, 0 },
-  { "-itype", 0, 15, 0 },
-  { "-iconic", "--iconic", 15, 0 },
-  { "-bg", "--background-color", 10, 1 },
-  { "-background", 0, 10, 1 },
-  { "-fg", "--foreground-color", 10, 1 },
-  { "-foreground", 0, 10, 1 },
-  { "-bd", "--border-color", 10, 1 },
-  { "-bw", "--border-width", 10, 1 },
-  { "-ib", "--internal-border", 10, 1 },
-  { "-ms", "--mouse-color", 10, 1 },
-  { "-cr", "--cursor-color", 10, 1 },
-  { "-fn", "--font", 10, 1 },
-  { "-font", 0, 10, 1 },
-  { "-g", "--geometry", 10, 1 },
-  { "-geometry", 0, 10, 1 },
-  { "-T", "--title", 10, 1 },
-  { "-title", 0, 10, 1 },
-  { "-name", "--name", 10, 1 },
-  { "-xrm", "--xrm", 10, 1 },
-  { "-r", "--reverse-video", 5, 0 },
-  { "-rv", 0, 5, 0 },
-  { "-reverse", 0, 5, 0 },
-  { "-hb", "--horizontal-scroll-bars", 5, 0 },
-  { "-vb", "--vertical-scroll-bars", 5, 0 },
-  /* These have the same priority as ordinary file name args,
-     so they are not reordered with respect to those.  */
-  { "-L", "--directory", 0, 1 },
-  { "-directory", 0, 0, 1 },
-  { "-l", "--load", 0, 1 },
-  { "-load", 0, 0, 1 },
-  { "-f", "--funcall", 0, 1 },
-  { "-funcall", 0, 0, 1 },
-  { "-eval", "--eval", 0, 1 },
-  { "-insert", "--insert", 0, 1 },
-  /* This should be processed after ordinary file name args and the like.  */
-  { "-kill", "--kill", -10, 0 },
-};
-
-/* Reorder the elements of ARGV (assumed to have ARGC elements)
-   so that the highest priority ones come first.
-   Do not change the order of elements of equal priority.
-   If an option takes an argument, keep it and its argument together.  */
-
-static void
-sort_args (argc, argv)
-     int argc;
-     char **argv;
-{
-  char **new = (char **) xmalloc (sizeof (char *) * argc);
-  /* For each element of argv,
-     the corresponding element of options is:
-     0 for an option that takes no arguments,
-     1 for an option that takes one argument, etc.
-     -1 for an ordinary non-option argument.  */
-  int *options = (int *) xmalloc (sizeof (int) * argc);
-  int *priority = (int *) xmalloc (sizeof (int) * argc);
-  int to = 1;
-  int from;
-  int i;
-
-  /* Categorize all the options,
-     and figure out which argv elts are option arguments.  */
-  for (from = 1; from < argc; from++)
-    {
-      options[from] = -1;
-      priority[from] = 0;
-      if (argv[from][0] == '-')
-	{
-	  int match, thislen;
-	  char *equals;
-
-	  /* Look for a match with a known old-fashioned option.  */
-	  for (i = 0; i < sizeof (standard_args) / sizeof (standard_args[0]); i++)
-	    if (!strcmp (argv[from], standard_args[i].name))
-	      {
-		options[from] = standard_args[i].nargs;
-		priority[from] = standard_args[i].priority;
-		if (from + standard_args[i].nargs >= argc)
-		  fatal ("Option `%s' requires an argument\n", argv[from]);
-		from += standard_args[i].nargs;
-		goto done;
-	      }
-
-	  /* Look for a match with a known long option.
-	     MATCH is -1 if no match so far, -2 if two or more matches so far,
-	     >= 0 (the table index of the match) if just one match so far.  */
-	  if (argv[from][1] == '-')
-	    {
-	      match = -1;
-	      thislen = strlen (argv[from]);
-	      equals = index (argv[from], '=');
-	      if (equals != 0)
-		thislen = equals - argv[from];
-
-	      for (i = 0;
-		   i < sizeof (standard_args) / sizeof (standard_args[0]); i++)
-		if (standard_args[i].longname
-		    && !strncmp (argv[from], standard_args[i].longname,
-				 thislen))
-		  {
-		    if (match == -1)
-		      match = i;
-		    else
-		      match = -2;
-		  }
-
-	      /* If we found exactly one match, use that.  */
-	      if (match >= 0)
-		{
-		  options[from] = standard_args[match].nargs;
-		  priority[from] = standard_args[match].priority;
-		  /* If --OPTION=VALUE syntax is used,
-		     this option uses just one argv element.  */
-		  if (equals != 0)
-		    options[from] = 0;
-		  if (from + options[from] >= argc)
-		    fatal ("Option `%s' requires an argument\n", argv[from]);
-		  from += options[from];
-		}
-	    }
-	done: ;
-	}
-    }
-
-  /* Copy the arguments, in order of decreasing priority, to NEW.  */
-  new[0] = argv[0];
-  while (to < argc)
-    {
-      int best = -1;
-      int best_priority = -9999;
-
-      /* Find the highest priority remaining option.
-	 If several have equal priority, take the first of them.  */
-      for (from = 1; from < argc; from++)
-	{
-	  if (argv[from] != 0 && priority[from] > best_priority)
-	    {
-	      best_priority = priority[from];
-	      best = from;
-	    }
-	  /* Skip option arguments--they are tied to the options.  */
-	  if (options[from] > 0)
-	    from += options[from];
-	}
-	    
-      if (best < 0)
-	abort ();
-
-      /* Copy the highest priority remaining option, with its args, to NEW.  */
-      new[to++] = argv[best];
-      for (i = 0; i < options[best]; i++)
-	new[to++] = argv[best + i + 1];
-
-      /* Clear out this option in ARGV.  */
-      argv[best] = 0;
-      for (i = 0; i < options[best]; i++)
-	argv[best + i + 1] = 0;
-    }
-
-  bcopy (new, argv, sizeof (char *) * argc);
-}
-
 DEFUN ("kill-emacs", Fkill_emacs, Skill_emacs, 0, 1, "P",
   "Exit the Emacs job and kill it.\n\
 If ARG is an integer, return ARG as the exit program code.\n\
@@ -1376,13 +945,7 @@ all of which are called before Emacs is actually killed.")
 
   shut_down_emacs (0, 0, STRINGP (arg) ? arg : Qnil);
 
-  /* If we have an auto-save list file,
-     kill it because we are exiting Emacs deliberately (not crashing).
-     Do it after shut_down_emacs, which does an auto-save.  */
-  if (STRINGP (Vauto_save_list_file_name))
-    unlink (XSTRING (Vauto_save_list_file_name)->data);
-
-  exit (INTEGERP (arg) ? XINT (arg)
+  exit ((XTYPE (arg) == Lisp_Int) ? XINT (arg)
 #ifdef VMS
 	: 1
 #else
@@ -1462,29 +1025,27 @@ shut_down_emacs (sig, no_x, stuff)
   unrequest_sigio ();
   signal (SIGIO, SIG_IGN);
 #endif
-
-#ifdef WINDOWSNT
-  term_ntproc ();
-#endif
 }
 
 
 
 #ifndef CANNOT_DUMP
+/* Nothing like this can be implemented on an Apollo.
+   What a loss!  */
 
 #ifdef HAVE_SHM
 
 DEFUN ("dump-emacs-data", Fdump_emacs_data, Sdump_emacs_data, 1, 1, 0,
   "Dump current state of Emacs into data file FILENAME.\n\
 This function exists on systems that use HAVE_SHM.")
-  (filename)
-     Lisp_Object filename;
+  (intoname)
+     Lisp_Object intoname;
 {
-  extern char my_edata[];
+  extern int my_edata;
   Lisp_Object tem;
 
-  CHECK_STRING (filename, 0);
-  filename = Fexpand_file_name (filename, Qnil);
+  CHECK_STRING (intoname, 0);
+  intoname = Fexpand_file_name (intoname, Qnil);
 
   tem = Vpurify_flag;
   Vpurify_flag = Qnil;
@@ -1493,9 +1054,9 @@ This function exists on systems that use HAVE_SHM.")
   /* Tell malloc where start of impure now is */
   /* Also arrange for warnings when nearly out of space.  */
 #ifndef SYSTEM_MALLOC
-  memory_warnings (my_edata, malloc_warning);
+  memory_warnings (&my_edata, malloc_warning);
 #endif
-  map_out_data (XSTRING (filename)->data);
+  map_out_data (XSTRING (intoname)->data);
 
   Vpurify_flag = tem;
 
@@ -1512,60 +1073,48 @@ This is used in the file `loadup.el' when building Emacs.\n\
 Bind `command-line-processed' to nil before dumping,\n\
 if you want the dumped Emacs to process its command line\n\
 and announce itself normally when it is run.")
-  (filename, symfile)
-     Lisp_Object filename, symfile;
+  (intoname, symname)
+     Lisp_Object intoname, symname;
 {
-  extern char my_edata[];
+  extern int my_edata;
   Lisp_Object tem;
 
-  CHECK_STRING (filename, 0);
-  filename = Fexpand_file_name (filename, Qnil);
-  if (!NILP (symfile))
+  CHECK_STRING (intoname, 0);
+  intoname = Fexpand_file_name (intoname, Qnil);
+  if (!NILP (symname))
     {
-      CHECK_STRING (symfile, 0);
-      if (XSTRING (symfile)->size)
-	symfile = Fexpand_file_name (symfile, Qnil);
+      CHECK_STRING (symname, 0);
+      if (XSTRING (symname)->size)
+	symname = Fexpand_file_name (symname, Qnil);
     }
 
   tem = Vpurify_flag;
   Vpurify_flag = Qnil;
 
-#ifdef HAVE_TZSET
-  set_time_zone_rule (dump_tz);
-#ifndef LOCALTIME_CACHE
-  /* Force a tz reload, since set_time_zone_rule doesn't.  */
-  tzset ();
-#endif
-#endif
-
   fflush (stdout);
 #ifdef VMS
-  mapout_data (XSTRING (filename)->data);
+  mapout_data (XSTRING (intoname)->data);
 #else
   /* Tell malloc where start of impure now is */
   /* Also arrange for warnings when nearly out of space.  */
 #ifndef SYSTEM_MALLOC
-#ifndef WINDOWSNT
-  /* On Windows, this was done before dumping, and that once suffices.
-     Meanwhile, my_edata is not valid on Windows.  */
-  memory_warnings (my_edata, malloc_warning);
-#endif /* not WINDOWSNT */
+  memory_warnings (&my_edata, malloc_warning);
 #endif
 /* 92.11.4 by M.Higashida */
 #if defined(MSDOS) && defined(EMX)
   {
-    int fd = open (XSTRING (filename)->data,
+    int fd = open (XSTRING (intoname)->data,
                    O_WRONLY|O_CREAT|O_TRUNC, S_IREAD|S_IWRITE);
     if (!fd) {
-      error ("Failure operating on %s", XSTRING (filename)->data);
+      error ("Failure operating on %s", XSTRING (intoname)->data);
     } else {
       _core (fd);
       close (fd);
     }
   }
 #else /* not MSDOS and EMX */
-  unexec (XSTRING (filename)->data,
-	  !NILP (symfile) ? XSTRING (symfile)->data : 0, my_edata, 0, 0);
+  unexec (XSTRING (intoname)->data,
+	  !NILP (symname) ? XSTRING (symname)->data : 0, &my_edata, 0, 0);
 #endif /* not MSDOS and EMX */
 /* end of patch */
 #endif /* not VMS */
@@ -1612,8 +1161,7 @@ decode_env_path (evarname, defalt)
     {
       p = index (path, SEPCHAR);
       if (!p) p = path + strlen (path);
-      lpath = Fcons (p - path ? make_string (path, p - path)
-		     : build_string ("."),
+      lpath = Fcons (p - path ? make_string (path, p - path) : Qnil,
 		     lpath);
       if (*p)
 	path = p + 1;
@@ -1649,10 +1197,6 @@ syms_of_emacs ()
     "Value is string indicating configuration Emacs was built for.");
   Vsystem_configuration = build_string (EMACS_CONFIGURATION);
 
-  DEFVAR_LISP ("system-configuration-options", &Vsystem_configuration_options,
-    "String containing the configuration options Emacs was built with.");
-  Vsystem_configuration_options = build_string (EMACS_CONFIG_OPTIONS);
-
   DEFVAR_BOOL ("noninteractive", &noninteractive1,
     "Non-nil means Emacs is running without interactive terminal.");
 
@@ -1660,8 +1204,7 @@ syms_of_emacs ()
     "Hook to be run whenever kill-emacs is called.\n\
 Since kill-emacs may be invoked when the terminal is disconnected (or\n\
 in other similar situations), functions placed on this hook should not\n\
-expect to be able to interact with the user.  To ask for confirmation,\n\
-see `kill-emacs-query-functions' instead.");
+expect to be able to interact with the user.");
   Vkill_emacs_hook = Qnil;
 
   DEFVAR_INT ("emacs-priority", &emacs_priority,
