@@ -1,11 +1,11 @@
 /* Evaluator for GNU Emacs Lisp interpreter.
-   Copyright (C) 1985, 1986, 1987, 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1987, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 1, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* 91.10.30 modified for Nemacs Ver.4.0.0 by K.Handa <handa@etl.go.jp> */
 /* 92.3.6   modified for Mule Ver.0.9.0 by K.Handa <handa@etl.go.jp> */
@@ -95,6 +94,7 @@ Lisp_Object Qinhibit_quit, Vinhibit_quit, Vquit_flag;
 Lisp_Object Qmocklisp_arguments, Vmocklisp_arguments, Qmocklisp;
 Lisp_Object Qand_rest, Qand_optional;
 Lisp_Object Qdebug_on_error;
+Lisp_Object Qload;		/* 91.11.1 by K.Handa */
 
 Lisp_Object Vrun_hooks;
 
@@ -134,10 +134,6 @@ Lisp_Object Vstack_trace_on_error;
    if an error is handled by the command loop's error handler.  */
 Lisp_Object Vdebug_on_error;
 
-/* List of conditions and regexps specifying error messages which
-   do not enter the debugger even if Vdebug_on_errors says they should.  */
-Lisp_Object Vdebug_ignored_errors;
-
 /* Nonzero means enter debugger if a quit signal
    is handled by the command loop's error handler. */
 int debug_on_quit;
@@ -154,8 +150,6 @@ Lisp_Object Vdebugger;
 
 void specbind (), record_unwind_protect ();
 
-Lisp_Object run_hook_with_args ();
-
 Lisp_Object funcall_lambda ();
 extern Lisp_Object ml_apply (); /* Apply a mocklisp function to unevaluated argument list */
 
@@ -163,11 +157,8 @@ init_eval_once ()
 {
   specpdl_size = 50;
   specpdl = (struct specbinding *) xmalloc (specpdl_size * sizeof (struct specbinding));
-  specpdl_ptr = specpdl;
   max_specpdl_size = 600;
   max_lisp_eval_depth = 300;	/* 92.7.10 by K.Handa - to compile regexp.el */
-
-  Vrun_hooks = Qnil;
 }
 
 init_eval ()
@@ -336,7 +327,7 @@ DEFUN ("progn", Fprogn, Sprogn, 0, UNEVALLED, 0,
   if (!EQ (Vmocklisp_arguments, Qt))
     {
       val = make_number (0);
-      while (!NILP (args) && (tem = Fcar (args), SYMBOLP (tem)))
+      while (!NILP (args) && (tem = Fcar (args), XTYPE (tem) == Lisp_Symbol))
 	{
 	  QUIT;
 	  specbind (tem, val), args = Fcdr (args);
@@ -498,7 +489,8 @@ and input is currently coming from the keyboard (not in keyboard macro).")
   /* If this isn't a byte-compiled function, there may be a frame at
      the top for Finteractive_p itself.  If so, skip it.  */
   fun = Findirect_function (*btp->function);
-  if (SUBRP (fun) && XSUBR (fun) == &Sinteractive_p)
+  if (XTYPE (fun) == Lisp_Subr
+      && (struct Lisp_Subr *) XPNTR (fun) == &Sinteractive_p)
     btp = btp->next;
 
   /* If we're running an Emacs 18-style byte-compiled function, there
@@ -520,7 +512,7 @@ and input is currently coming from the keyboard (not in keyboard macro).")
      Fbytecode at the top.  If this frame is for a built-in function
      (such as load or eval-region) return nil.  */
   fun = Findirect_function (*btp->function);
-  if (SUBRP (fun))
+  if (XTYPE (fun) == Lisp_Subr)
     return Qnil;
   /* btp points to the frame of a Lisp function that called interactive-p.
      Return t if that function was called interactively.  */
@@ -585,23 +577,19 @@ If INITVALUE is missing, SYMBOL's value is not set.")
   (args)
      Lisp_Object args;
 {
-  register Lisp_Object sym, tem, tail;
+  register Lisp_Object sym, tem;
 
   sym = Fcar (args);
-  tail = Fcdr (args);
-  if (!NILP (Fcdr (Fcdr (tail))))
-    error ("too many arguments");
-
-  if (!NILP (tail))
+  tem = Fcdr (args);
+  if (!NILP (tem))
     {
       tem = Fdefault_boundp (sym);
       if (NILP (tem))
 	Fset_default (sym, Feval (Fcar (Fcdr (args))));
     }
-  tail = Fcdr (Fcdr (args));
-  if (!NILP (Fcar (tail)))
+  tem = Fcar (Fcdr (Fcdr (args)));
+  if (!NILP (tem))
     {
-      tem = Fcar (tail);
       if (!NILP (Vpurify_flag))
 	tem = Fpurecopy (tem);
       Fput (sym, Qvariable_documentation, tem);
@@ -630,9 +618,6 @@ it would override the user's choice.")
   register Lisp_Object sym, tem;
 
   sym = Fcar (args);
-  if (!NILP (Fcdr (Fcdr (Fcdr (args)))))
-    error ("too many arguments");
-
   Fset_default (sym, Feval (Fcar (Fcdr (args))));
   tem = Fcar (Fcdr (Fcdr (args)));
   if (!NILP (tem))
@@ -649,23 +634,17 @@ DEFUN ("user-variable-p", Fuser_variable_p, Suser_variable_p, 1, 1, 0,
   "Returns t if VARIABLE is intended to be set and modified by users.\n\
 \(The alternative is a variable used internally in a Lisp program.)\n\
 Determined by whether the first character of the documentation\n\
-for the variable is `*'.")
+for the variable is \"*\"")
   (variable)
      Lisp_Object variable;
 {
   Lisp_Object documentation;
   
   documentation = Fget (variable, Qvariable_documentation);
-  if (INTEGERP (documentation) && XINT (documentation) < 0)
+  if (XTYPE (documentation) == Lisp_Int && XINT (documentation) < 0)
     return Qt;
-  if (STRINGP (documentation)
-      && ((unsigned char) XSTRING (documentation)->data[0] == '*'))
-    return Qt;
-  /* If it is (STRING . INTEGER), a negative integer means a user variable.  */
-  if (CONSP (documentation)
-      && STRINGP (XCONS (documentation)->car)
-      && INTEGERP (XCONS (documentation)->cdr)
-      && XINT (XCONS (documentation)->cdr) < 0)
+  if ((XTYPE (documentation) == Lisp_String) &&
+      ((unsigned char) XSTRING (documentation)->data[0] == '*'))
     return Qt;
   return Qnil;
 }  
@@ -690,7 +669,7 @@ Each VALUEFORM can refer to the symbols already bound by this VARLIST.")
     {
       QUIT;
       elt = Fcar (varlist);
-      if (SYMBOLP (elt))
+      if (XTYPE (elt) == Lisp_Symbol)
 	specbind (elt, Qnil);
       else if (! NILP (Fcdr (Fcdr (elt))))
 	Fsignal (Qerror,
@@ -738,7 +717,7 @@ All the VALUEFORMs are evalled before any symbols are bound.")
     {
       QUIT;
       elt = Fcar (varlist);
-      if (SYMBOLP (elt))
+      if (XTYPE (elt) == Lisp_Symbol)
 	temps [argnum++] = Qnil;
       else if (! NILP (Fcdr (Fcdr (elt))))
 	Fsignal (Qerror,
@@ -755,7 +734,7 @@ All the VALUEFORMs are evalled before any symbols are bound.")
     {
       elt = Fcar (varlist);
       tem = temps[argnum++];
-      if (SYMBOLP (elt))
+      if (XTYPE (elt) == Lisp_Symbol)
 	specbind (elt, tem);
       else
 	specbind (Fcar (elt), tem);
@@ -797,9 +776,9 @@ Otherwise, the macro is expanded and the expansion is considered\n\
 in place of FORM.  When a non-macro-call results, it is returned.\n\n\
 The second optional arg ENVIRONMENT species an environment of macro\n\
 definitions to shadow the loaded ones for use in file byte-compilation.")
-  (form, environment)
+  (form, env)
      register Lisp_Object form;
-     Lisp_Object environment;
+     Lisp_Object env;
 {
   /* With cleanups from Hallvard Furuseth.  */
   register Lisp_Object expander, sym, def, tem;
@@ -808,18 +787,18 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
     {
       /* Come back here each time we expand a macro call,
 	 in case it expands into another macro call.  */
-      if (!CONSP (form))
+      if (XTYPE (form) != Lisp_Cons)
 	break;
       /* Set SYM, give DEF and TEM right values in case SYM is not a symbol. */
       def = sym = XCONS (form)->car;
       tem = Qnil;
       /* Trace symbols aliases to other symbols
 	 until we get a symbol that is not an alias.  */
-      while (SYMBOLP (def))
+      while (XTYPE (def) == Lisp_Symbol)
 	{
 	  QUIT;
 	  sym = def;
-	  tem = Fassq (sym, environment);
+	  tem = Fassq (sym, env);
 	  if (NILP (tem))
 	    {
 	      def = XSYMBOL (sym)->function;
@@ -828,13 +807,14 @@ definitions to shadow the loaded ones for use in file byte-compilation.")
 	    }
 	  break;
 	}
-      /* Right now TEM is the result from SYM in ENVIRONMENT,
+      /* Right now TEM is the result from SYM in ENV,
 	 and if TEM is nil then DEF is SYM's function definition.  */
       if (NILP (tem))
 	{
-	  /* SYM is not mentioned in ENVIRONMENT.
+	  /* SYM is not mentioned in ENV.
 	     Look at its function definition.  */
-	  if (EQ (def, Qunbound) || !CONSP (def))
+	  if (EQ (def, Qunbound)
+	      || XTYPE (def) != Lisp_Cons)
 	    /* Not defined or definition not suitable */
 	    break;
 	  if (EQ (XCONS (def)->car, Qautoload))
@@ -968,8 +948,8 @@ unwind_to_catch (catch, value)
 DEFUN ("throw", Fthrow, Sthrow, 2, 2, 0,
   "(throw TAG VALUE): throw to the catch for TAG and return VALUE from it.\n\
 Both TAG and VALUE are evalled.")
-  (tag, value)
-     register Lisp_Object tag, value;
+  (tag, val)
+     register Lisp_Object tag, val;
 {
   register struct catchtag *c;
 
@@ -979,9 +959,9 @@ Both TAG and VALUE are evalled.")
 	for (c = catchlist; c; c = c->next)
 	  {
 	    if (EQ (c->tag, tag))
-	      unwind_to_catch (c, value);
+	      unwind_to_catch (c, val);
 	  }
-      tag = Fsignal (Qno_catch, Fcons (tag, Fcons (value, Qnil)));
+      tag = Fsignal (Qno_catch, Fcons (tag, Fcons (val, Qnil)));
     }
 }
 
@@ -1090,16 +1070,6 @@ See also the function `signal' for more info.")
   return val;
 }
 
-/* Call the function BFUN with no arguments, catching errors within it
-   according to HANDLERS.  If there is an error, call HFUN with
-   one argument which is the data that describes the error:
-   (SIGNALNAME . DATA)
-
-   HANDLERS can be a list of conditions to catch.
-   If HANDLERS is Qt, catch all errors.
-   If HANDLERS is Qerror, catch all errors
-   but allow the debugger to run if that is enabled.  */
-
 Lisp_Object
 internal_condition_case (bfun, handlers, hfun)
      Lisp_Object (*bfun) ();
@@ -1110,11 +1080,6 @@ internal_condition_case (bfun, handlers, hfun)
   struct catchtag c;
   struct handler h;
 
-  /* Since Fsignal resets this to 0, it had better be 0 now
-     or else we have a potential bug.  */
-  if (interrupt_input_blocked != 0)
-    abort ();
-
   c.tag = Qnil;
   c.val = Qnil;
   c.backlist = backtrace_list;
@@ -1141,8 +1106,6 @@ internal_condition_case (bfun, handlers, hfun)
   return val;
 }
 
-/* Like internal_condition_case but call HFUN with ARG as its argument.  */
-
 Lisp_Object
 internal_condition_case_1 (bfun, arg, handlers, hfun)
      Lisp_Object (*bfun) ();
@@ -1207,7 +1170,7 @@ See also the function `condition-case'.")
   if (gc_in_progress || waiting_for_input)
     abort ();
 
-#ifdef HAVE_WINDOW_SYSTEM
+#ifdef HAVE_X_WINDOWS
   TOTALLY_UNBLOCK_INPUT;
 #endif
 
@@ -1228,8 +1191,8 @@ See also the function `condition-case'.")
 #else
       if (EQ (clause, Qlambda))
 	{
-	  /* We can't return values to code which signaled an error, but we
-	     can continue code which has signaled a quit.  */
+	  /* We can't return values to code which signalled an error, but we
+	     can continue code which has signalled a quit.  */
 	  if (EQ (error_symbol, Qquit))
 	    return Qnil;
 	  else
@@ -1243,7 +1206,7 @@ See also the function `condition-case'.")
 	  struct handler *h = handlerlist;
 
 	  handlerlist = allhandlers;
-	  if (EQ (data, memory_signal_data))
+	  if (data == memory_signal_data)
 	    unwind_data = memory_signal_data;
 	  else
 	    unwind_data = Fcons (error_symbol, data);
@@ -1283,45 +1246,6 @@ wants_debugger (list, conditions)
   return 0;
 }
 
-/* Return 1 if an error with condition-symbols CONDITIONS,
-   and described by SIGNAL-DATA, should skip the debugger
-   according to debugger-ignore-errors.  */
-
-static int
-skip_debugger (conditions, data)
-     Lisp_Object conditions, data;
-{
-  Lisp_Object tail;
-  int first_string = 1;
-  Lisp_Object error_message;
-
-  for (tail = Vdebug_ignored_errors; CONSP (tail);
-       tail = XCONS (tail)->cdr)
-    {
-      if (STRINGP (XCONS (tail)->car))
-	{
-	  if (first_string)
-	    {
-	      error_message = Ferror_message_string (data);
-	      first_string = 0;
-	    }
-	  if (fast_string_match (XCONS (tail)->car, error_message) >= 0)
-	    return 1;
-	}
-      else
-	{
-	  Lisp_Object contail;
-
-	  for (contail = conditions; CONSP (contail);
-	       contail = XCONS (contail)->cdr)
-	    if (EQ (XCONS (tail)->car, XCONS (contail)->car))
-	      return 1;
-	}
-    }
-
-  return 0;
-}
-
 /* Value of Qlambda means we have called debugger and user has continued.
    Store value returned from debugger into *DEBUGGER_VALUE_PTR.  */
 
@@ -1342,15 +1266,14 @@ find_handler_clause (handlers, conditions, sig, data, debugger_value_ptr)
       if ((EQ (sig, Qquit)
 	   ? debug_on_quit
 	   : wants_debugger (Vdebug_on_error, conditions))
-	  && ! skip_debugger (conditions, Fcons (sig, data))
 	  && when_entered_debugger < num_nonmacro_input_chars)
 	{
 	  int count = specpdl_ptr - specpdl;
 	  specbind (Qdebug_on_error, Qnil);
-	  *debugger_value_ptr
-	    = call_debugger (Fcons (Qerror,
-				    Fcons (Fcons (sig, data),
-					   Qnil)));
+	  *debugger_value_ptr =
+	    call_debugger (Fcons (Qerror,
+				  Fcons (Fcons (sig, data),
+					 Qnil)));
 	  return unbind_to (count, Qlambda);
 	}
       return Qt;
@@ -1457,7 +1380,7 @@ Also, a symbol satisfies `commandp' if its function definition does so.")
 
   /* Emacs primitives are interactive if their DEFUN specifies an
      interactive spec.  */
-  if (SUBRP (fun))
+  if (XTYPE (fun) == Lisp_Subr)
     {
       if (XSUBR (fun)->prompt)
 	return Qt;
@@ -1468,19 +1391,20 @@ Also, a symbol satisfies `commandp' if its function definition does so.")
   /* Bytecode objects are interactive if they are long enough to
      have an element whose index is COMPILED_INTERACTIVE, which is
      where the interactive spec is stored.  */
-  else if (COMPILEDP (fun))
-    return ((XVECTOR (fun)->size & PSEUDOVECTOR_SIZE_MASK) > COMPILED_INTERACTIVE
+  else if (XTYPE (fun) == Lisp_Compiled)
+    return (XVECTOR (fun)->size > COMPILED_INTERACTIVE
 	    ? Qt : Qnil);
 
   /* Strings and vectors are keyboard macros.  */
-  if (STRINGP (fun) || VECTORP (fun))
+  if (XTYPE (fun) == Lisp_String
+      || XTYPE (fun) == Lisp_Vector)
     return Qt;
 
   /* Lists may represent commands.  */
   if (!CONSP (fun))
     return Qnil;
   funcar = Fcar (fun);
-  if (!SYMBOLP (funcar))
+  if (XTYPE (funcar) != Lisp_Symbol)
     return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
   if (EQ (funcar, Qlambda))
     return Fassq (Qinteractive, Fcdr (Fcdr (fun)));
@@ -1518,7 +1442,7 @@ this does nothing and returns nil.")
 
   /* If function is defined and not as an autoload, don't override */
   if (!EQ (XSYMBOL (function)->function, Qunbound)
-      && !(CONSP (XSYMBOL (function)->function)
+      && !(XTYPE (XSYMBOL (function)->function) == Lisp_Cons
 	   && EQ (XCONS (XSYMBOL (function)->function)->car, Qautoload)))
     return Qnil;
 
@@ -1570,7 +1494,8 @@ do_autoload (fundef, funname)
   /* Value saved here is to be restored into Vautoload_queue */
   record_unwind_protect (un_autoload, Vautoload_queue);
   Vautoload_queue = Qt;
-  Fload (Fcar (Fcdr (fundef)), Qnil, noninteractive ? Qt : Qnil, Qnil);
+				/* 91.10.30 by K.Handa */
+  call4 (Qload, Fcar (Fcdr (fundef)), Qnil, noninteractive ? Qt : Qnil, Qnil);
 
   /* Save the old autoloads, in case we ever do an unload. */
   queue = Vautoload_queue;
@@ -1610,15 +1535,15 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
   struct backtrace backtrace;
   struct gcpro gcpro1, gcpro2, gcpro3;
 
-  if (SYMBOLP (form))
+  if (XTYPE (form) == Lisp_Symbol)
     {
       if (EQ (Vmocklisp_arguments, Qt))
         return Fsymbol_value (form);
       val = Fsymbol_value (form);
       if (NILP (val))
-	XSETFASTINT (val, 0);
+	XFASTINT (val) = 0;
       else if (EQ (val, Qt))
-	XSETFASTINT (val, 1);
+	XFASTINT (val) = 1;
       return val;
     }
   if (!CONSP (form))
@@ -1659,7 +1584,7 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
  retry:
   fun = Findirect_function (original_fun);
 
-  if (SUBRP (fun))
+  if (XTYPE (fun) == Lisp_Subr)
     {
       Lisp_Object numargs;
       Lisp_Object argvals[7];
@@ -1764,14 +1689,14 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
 	  abort ();
 	}
     }
-  if (COMPILEDP (fun))
+  if (XTYPE (fun) == Lisp_Compiled)
     val = apply_lambda (fun, original_args, 1);
   else
     {
       if (!CONSP (fun))
 	return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       funcar = Fcar (fun);
-      if (!SYMBOLP (funcar))
+      if (XTYPE (funcar) != Lisp_Symbol)
 	return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       if (EQ (funcar, Qautoload))
 	{
@@ -1791,9 +1716,9 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
   if (!EQ (Vmocklisp_arguments, Qt))
     {
       if (NILP (val))
-	XSETFASTINT (val, 0);
+	XFASTINT (val) = 0;
       else if (EQ (val, Qt))
-	XSETFASTINT (val, 1);
+	XFASTINT (val) = 1;
     }
   lisp_eval_depth--;
   if (backtrace.debug_on_exit)
@@ -1804,7 +1729,6 @@ DEFUN ("eval", Feval, Seval, 1, 1, 0,
 
 DEFUN ("apply", Fapply, Sapply, 2, MANY, 0,
   "Call FUNCTION with our remaining args, using our last arg as list of args.\n\
-Then return the value FUNCTION returns.\n\
 Thus, (apply '+ 1 2 '(3 4)) returns 10.")
   (nargs, args)
      int nargs;
@@ -1841,7 +1765,7 @@ Thus, (apply '+ 1 2 '(3 4)) returns 10.")
       goto funcall;
     }
 
-  if (SUBRP (fun))
+  if (XTYPE (fun) == Lisp_Subr)
     {
       if (numargs < XSUBR (fun)->min_args
 	  || (XSUBR (fun)->max_args >= 0 && XSUBR (fun)->max_args < numargs))
@@ -1882,233 +1806,6 @@ Thus, (apply '+ 1 2 '(3 4)) returns 10.")
   RETURN_UNGCPRO (Ffuncall (gcpro1.nvars, funcall_args));
 }
 
-/* Run hook variables in various ways.  */
-
-enum run_hooks_condition {to_completion, until_success, until_failure};
-
-DEFUN ("run-hooks", Frun_hooks, Srun_hooks, 1, MANY, 0,
-  "Run each hook in HOOKS.  Major mode functions use this.\n\
-Each argument should be a symbol, a hook variable.\n\
-These symbols are processed in the order specified.\n\
-If a hook symbol has a non-nil value, that value may be a function\n\
-or a list of functions to be called to run the hook.\n\
-If the value is a function, it is called with no arguments.\n\
-If it is a list, the elements are called, in order, with no arguments.\n\
-\n\
-To make a hook variable buffer-local, use `make-local-hook',\n\
-not `make-local-variable'.")
-  (nargs, args)
-     int nargs;
-     Lisp_Object *args;
-{
-  Lisp_Object hook[1];
-  register int i;
-
-  for (i = 0; i < nargs; i++)
-    {
-      hook[0] = args[i];
-      run_hook_with_args (1, hook, to_completion);
-    }
-
-  return Qnil;
-}
-      
-DEFUN ("run-hook-with-args",
-  Frun_hook_with_args, Srun_hook_with_args, 1, MANY, 0,
-  "Run HOOK with the specified arguments ARGS.\n\
-HOOK should be a symbol, a hook variable.  If HOOK has a non-nil\n\
-value, that value may be a function or a list of functions to be\n\
-called to run the hook.  If the value is a function, it is called with\n\
-the given arguments and its return value is returned.  If it is a list\n\
-of functions, those functions are called, in order,\n\
-with the given arguments ARGS.\n\
-It is best not to depend on the value return by `run-hook-with-args',\n\
-as that may change.\n\
-\n\
-To make a hook variable buffer-local, use `make-local-hook',\n\
-not `make-local-variable'.")
-  (nargs, args)
-     int nargs;
-     Lisp_Object *args;
-{
-  return run_hook_with_args (nargs, args, to_completion);
-}
-
-DEFUN ("run-hook-with-args-until-success",
-  Frun_hook_with_args_until_success, Srun_hook_with_args_until_success,
-  1, MANY, 0,
-  "Run HOOK with the specified arguments ARGS.\n\
-HOOK should be a symbol, a hook variable.  Its value should\n\
-be a list of functions.  We call those functions, one by one,\n\
-passing arguments ARGS to each of them, until one of them\n\
-returns a non-nil value.  Then we return that value.\n\
-If all the functions return nil, we return nil.\n\
-\n\
-To make a hook variable buffer-local, use `make-local-hook',\n\
-not `make-local-variable'.")
-  (nargs, args)
-     int nargs;
-     Lisp_Object *args;
-{
-  return run_hook_with_args (nargs, args, until_success);
-}
-
-DEFUN ("run-hook-with-args-until-failure",
-  Frun_hook_with_args_until_failure, Srun_hook_with_args_until_failure,
-  1, MANY, 0,
-  "Run HOOK with the specified arguments ARGS.\n\
-HOOK should be a symbol, a hook variable.  Its value should\n\
-be a list of functions.  We call those functions, one by one,\n\
-passing arguments ARGS to each of them, until one of them\n\
-returns nil.  Then we return nil.\n\
-If all the functions return non-nil, we return non-nil.\n\
-\n\
-To make a hook variable buffer-local, use `make-local-hook',\n\
-not `make-local-variable'.")
-  (nargs, args)
-     int nargs;
-     Lisp_Object *args;
-{
-  return run_hook_with_args (nargs, args, until_failure);
-}
-
-/* ARGS[0] should be a hook symbol.
-   Call each of the functions in the hook value, passing each of them
-   as arguments all the rest of ARGS (all NARGS - 1 elements).
-   COND specifies a condition to test after each call
-   to decide whether to stop.
-   The caller (or its caller, etc) must gcpro all of ARGS,
-   except that it isn't necessary to gcpro ARGS[0].  */
-
-Lisp_Object
-run_hook_with_args (nargs, args, cond)
-     int nargs;
-     Lisp_Object *args;
-     enum run_hooks_condition cond;
-{
-  Lisp_Object sym, val, ret;
-  struct gcpro gcpro1, gcpro2;
-
-  /* If we are dying or still initializing,
-     don't do anything--it would probably crash if we tried.  */
-  if (NILP (Vrun_hooks))
-    return;
-
-  sym = args[0];
-  val = find_symbol_value (sym);
-  ret = (cond == until_failure ? Qt : Qnil);
-
-  if (EQ (val, Qunbound) || NILP (val))
-    return ret;
-  else if (!CONSP (val) || EQ (XCONS (val)->car, Qlambda))
-    {
-      args[0] = val;
-      return Ffuncall (nargs, args);
-    }
-  else
-    {
-      GCPRO2 (sym, val);
-
-      for (;
-	   CONSP (val) && ((cond == to_completion)
-			   || (cond == until_success ? NILP (ret)
-			       : !NILP (ret)));
-	   val = XCONS (val)->cdr)
-	{
-	  if (EQ (XCONS (val)->car, Qt))
-	    {
-	      /* t indicates this hook has a local binding;
-		 it means to run the global binding too.  */
-	      Lisp_Object globals;
-
-	      for (globals = Fdefault_value (sym);
-		   CONSP (globals) && ((cond == to_completion)
-				       || (cond == until_success ? NILP (ret)
-					   : !NILP (ret)));
-		   globals = XCONS (globals)->cdr)
-		{
-		  args[0] = XCONS (globals)->car;
-		  /* In a global value, t should not occur.  If it does, we
-		     must ignore it to avoid an endless loop.  */
-		  if (!EQ (args[0], Qt))
-		    ret = Ffuncall (nargs, args);
-		}
-	    }
-	  else
-	    {
-	      args[0] = XCONS (val)->car;
-	      ret = Ffuncall (nargs, args);
-	    }
-	}
-
-      UNGCPRO;
-      return ret;
-    }
-}
-
-/* Run a hook symbol ARGS[0], but use FUNLIST instead of the actual
-   present value of that symbol.
-   Call each element of FUNLIST,
-   passing each of them the rest of ARGS.
-   The caller (or its caller, etc) must gcpro all of ARGS,
-   except that it isn't necessary to gcpro ARGS[0].  */
-
-Lisp_Object
-run_hook_list_with_args (funlist, nargs, args)
-     Lisp_Object funlist;
-     int nargs;
-     Lisp_Object *args;
-{
-  Lisp_Object sym;
-  Lisp_Object val;
-  struct gcpro gcpro1, gcpro2;
-
-  sym = args[0];
-  GCPRO2 (sym, val);
-
-  for (val = funlist; CONSP (val); val = XCONS (val)->cdr)
-    {
-      if (EQ (XCONS (val)->car, Qt))
-	{
-	  /* t indicates this hook has a local binding;
-	     it means to run the global binding too.  */
-	  Lisp_Object globals;
-
-	  for (globals = Fdefault_value (sym);
-	       CONSP (globals);
-	       globals = XCONS (globals)->cdr)
-	    {
-	      args[0] = XCONS (globals)->car;
-	      /* In a global value, t should not occur.  If it does, we
-		 must ignore it to avoid an endless loop.  */
-	      if (!EQ (args[0], Qt))
-		Ffuncall (nargs, args);
-	    }
-	}
-      else
-	{
-	  args[0] = XCONS (val)->car;
-	  Ffuncall (nargs, args);
-	}
-    }
-  UNGCPRO;
-  return Qnil;
-}
-
-/* Run the hook HOOK, giving each function the two args ARG1 and ARG2.  */
-
-void
-run_hook_with_args_2 (hook, arg1, arg2)
-     Lisp_Object hook, arg1, arg2;
-{
-  Lisp_Object temp[3];
-  temp[0] = hook;
-  temp[1] = arg1;
-  temp[2] = arg2;
-
-  Frun_hook_with_args (3, temp);
-}
-
 /* Apply fn to arg */
 Lisp_Object
 apply1 (fn, arg)
@@ -2288,7 +1985,6 @@ call6 (fn, arg1, arg2, arg3, arg4, arg5, arg6)
 
 DEFUN ("funcall", Ffuncall, Sfuncall, 1, MANY, 0,
   "Call first argument as a function, passing remaining arguments to it.\n\
-Return the value that function returns.\n\
 Thus, (funcall 'cons 'x 'y) returns (x . y).")
   (nargs, args)
      int nargs;
@@ -2332,12 +2028,12 @@ Thus, (funcall 'cons 'x 'y) returns (x . y).")
 
   fun = Findirect_function (fun);
 
-  if (SUBRP (fun))
+  if (XTYPE (fun) == Lisp_Subr)
     {
       if (numargs < XSUBR (fun)->min_args
 	  || (XSUBR (fun)->max_args >= 0 && XSUBR (fun)->max_args < numargs))
 	{
-	  XSETFASTINT (lisp_numargs, numargs);
+	  XFASTINT (lisp_numargs) = numargs;
 	  return Fsignal (Qwrong_number_of_arguments, Fcons (fun, Fcons (lisp_numargs, Qnil)));
 	}
 
@@ -2405,14 +2101,14 @@ Thus, (funcall 'cons 'x 'y) returns (x . y).")
 	  abort ();
 	}
     }
-  if (COMPILEDP (fun))
+  if (XTYPE (fun) == Lisp_Compiled)
     val = funcall_lambda (fun, numargs, args + 1);
   else
     {
       if (!CONSP (fun))
 	return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       funcar = Fcar (fun);
-      if (!SYMBOLP (funcar))
+      if (XTYPE (funcar) != Lisp_Symbol)
 	return Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       if (EQ (funcar, Qlambda))
 	val = funcall_lambda (fun, numargs, args + 1);
@@ -2499,11 +2195,11 @@ funcall_lambda (fun, nargs, arg_vector)
 
   specbind (Qmocklisp_arguments, Qt);   /* t means NOT mocklisp! */
 
-  XSETFASTINT (numargs, nargs);
+  XFASTINT (numargs) = nargs;
 
-  if (CONSP (fun))
+  if (XTYPE (fun) == Lisp_Cons)
     syms_left = Fcar (Fcdr (fun));
-  else if (COMPILEDP (fun))
+  else if (XTYPE (fun) == Lisp_Compiled)
     syms_left = XVECTOR (fun)->contents[COMPILED_ARGLIST];
   else abort ();
 
@@ -2512,7 +2208,7 @@ funcall_lambda (fun, nargs, arg_vector)
     {
       QUIT;
       next = Fcar (syms_left);
-      while (!SYMBOLP (next))
+      while (XTYPE (next) != Lisp_Symbol)
 	next = Fsignal (Qinvalid_function, Fcons (fun, Qnil));
       if (EQ (next, Qand_rest))
 	rest = 1;
@@ -2537,40 +2233,14 @@ funcall_lambda (fun, nargs, arg_vector)
   if (i < nargs)
     return Fsignal (Qwrong_number_of_arguments, Fcons (fun, Fcons (numargs, Qnil)));
 
-  if (CONSP (fun))
+  if (XTYPE (fun) == Lisp_Cons)
     val = Fprogn (Fcdr (Fcdr (fun)));
   else
-    {
-      /* If we have not actually read the bytecode string
-	 and constants vector yet, fetch them from the file.  */
-      if (CONSP (XVECTOR (fun)->contents[COMPILED_BYTECODE]))
-	Ffetch_bytecode (fun);
-      val = Fbyte_code (XVECTOR (fun)->contents[COMPILED_BYTECODE],
-			XVECTOR (fun)->contents[COMPILED_CONSTANTS],
-			XVECTOR (fun)->contents[COMPILED_STACK_DEPTH]);
-    }
+    val = Fbyte_code (XVECTOR (fun)->contents[COMPILED_BYTECODE],
+		      XVECTOR (fun)->contents[COMPILED_CONSTANTS],
+		      XVECTOR (fun)->contents[COMPILED_STACK_DEPTH]);
   return unbind_to (count, val);
 }
-
-DEFUN ("fetch-bytecode", Ffetch_bytecode, Sfetch_bytecode,
-  1, 1, 0,
-  "If byte-compiled OBJECT is lazy-loaded, fetch it now.")
-  (object)
-     Lisp_Object object;
-{
-  Lisp_Object tem;
-
-  if (COMPILEDP (object)
-      && CONSP (XVECTOR (object)->contents[COMPILED_BYTECODE]))
-    {
-      tem = read_doc_string (XVECTOR (object)->contents[COMPILED_BYTECODE]);
-      if (!CONSP (tem))
-	error ("invalid byte code");
-      XVECTOR (object)->contents[COMPILED_BYTECODE] = XCONS (tem)->car;
-      XVECTOR (object)->contents[COMPILED_CONSTANTS] = XCONS (tem)->cdr;
-    }
-  return object;
-}
 
 void
 grow_specpdl ()
@@ -2600,6 +2270,7 @@ void
 specbind (symbol, value)
      Lisp_Object symbol, value;
 {
+  extern void store_symval_forwarding (); /* in eval.c */
   Lisp_Object ovalue;
 
   CHECK_SYMBOL (symbol, 0);
@@ -2610,7 +2281,7 @@ specbind (symbol, value)
   specpdl_ptr->func = 0;
   specpdl_ptr->old_value = ovalue = find_symbol_value (symbol);
   specpdl_ptr++;
-  if (BUFFER_OBJFWDP (ovalue) || KBOARD_OBJFWDP (ovalue))
+  if (XTYPE (ovalue) == Lisp_Buffer_Objfwd)
     store_symval_forwarding (symbol, ovalue, value);
   else
     Fset (symbol, value);
@@ -2734,7 +2405,7 @@ Output stream used is value of `standard-output'.")
   extern Lisp_Object Vprint_level;
   struct gcpro gcpro1;
 
-  XSETFASTINT (Vprint_level, 3);
+  XFASTINT (Vprint_level) = 3;
 
   tail = Qnil;
   GCPRO1 (tail);
@@ -2781,7 +2452,7 @@ Output stream used is value of `standard-output'.")
 }
 
 DEFUN ("backtrace-frame", Fbacktrace_frame, Sbacktrace_frame, 1, 1, "",
-  "Return the function and arguments NFRAMES up from current execution point.\n\
+  "Return the function and arguments N frames up from current execution point.\n\
 If that frame has not evaluated the arguments yet (or is a special form),\n\
 the value is (nil FUNCTION ARG-FORMS...).\n\
 If that frame has evaluated its arguments and called its function already,\n\
@@ -2789,7 +2460,7 @@ the value is (t FUNCTION ARG-VALUES...).\n\
 A &rest arg is represented as the tail of the list ARG-VALUES.\n\
 FUNCTION is whatever was supplied as car of evaluated list,\n\
 or a lambda expression for macro calls.\n\
-If NFRAMES is more than the number of frames, the value is nil.")
+If N is more than the number of frames, the value is nil.")
   (nframes)
      Lisp_Object nframes;
 {
@@ -2838,7 +2509,7 @@ Typing C-g sets `quit-flag' non-nil, regardless of `inhibit-quit'.");
   DEFVAR_LISP ("inhibit-quit", &Vinhibit_quit,
     "Non-nil inhibits C-g quitting from happening immediately.\n\
 Note that `quit-flag' will still be set by typing C-g,\n\
-so a quit will be signaled as soon as `inhibit-quit' is nil.\n\
+so a quit will be signalled as soon as `inhibit-quit' is nil.\n\
 To prevent this happening, set `quit-flag' to nil\n\
 before making `inhibit-quit' nil.");
   Vinhibit_quit = Qnil;
@@ -2875,6 +2546,9 @@ before making `inhibit-quit' nil.");
   Qand_optional = intern ("&optional");
   staticpro (&Qand_optional);
 
+  Qload = intern ("load");	/* 91.11.1 by K.Handa */
+  staticpro (&Qload);		/* 91.11.1 by K.Handa */
+
   DEFVAR_LISP ("stack-trace-on-error", &Vstack_trace_on_error,
     "*Non-nil means automatically display a backtrace buffer\n\
 after any error that is handled by the editor command loop.\n\
@@ -2890,15 +2564,6 @@ if one of its condition symbols appears in the list.\n\
 See also variable `debug-on-quit'.");
   Vdebug_on_error = Qnil;
 
-  DEFVAR_LISP ("debug-ignored-errors", &Vdebug_ignored_errors,
-    "*List of errors for which the debugger should not be called.\n\
-Each element may be a condition-name or a regexp that matches error messages.\n\
-If any element applies to a given error, that error skips the debugger\n\
-and just returns to top level.\n\
-This overrides the variable `debug-on-error'.\n\
-It does not apply to errors handled by `condition-case'.");
-  Vdebug_ignored_errors = Qnil;
-
   DEFVAR_BOOL ("debug-on-quit", &debug_on_quit,
     "*Non-nil means enter debugger if quit is signaled (C-g, for example).\n\
 Does not apply if quit is handled by a `condition-case'.");
@@ -2925,6 +2590,7 @@ If due to `eval' entry, one arg, t.");
   DEFVAR_LISP ("run-hooks", &Vrun_hooks,
     "Set to the function `run-hooks', if that function has been defined.\n\
 Otherwise, nil (in a bare Emacs without preloaded Lisp code).");
+  Vrun_hooks = Qnil;
 
   staticpro (&Vautoload_queue);
   Vautoload_queue = Qnil;
@@ -2959,11 +2625,6 @@ Otherwise, nil (in a bare Emacs without preloaded Lisp code).");
   defsubr (&Seval);
   defsubr (&Sapply);
   defsubr (&Sfuncall);
-  defsubr (&Srun_hooks);
-  defsubr (&Srun_hook_with_args);
-  defsubr (&Srun_hook_with_args_until_success);
-  defsubr (&Srun_hook_with_args_until_failure);
-  defsubr (&Sfetch_bytecode);
   defsubr (&Sbacktrace_debug);
   defsubr (&Sbacktrace);
   defsubr (&Sbacktrace_frame);
