@@ -671,7 +671,7 @@ This is the same as the exponent of a float.")
      Lisp_Object arg;
 {
   Lisp_Object val;
-  int value;
+  Lisp_Object_Int value;
   double f = extract_float (arg);
 
   if (f == 0.0)
