@@ -37,5 +37,5 @@ extern FONT_INFO *font_table;
 extern FONTSET_INFO *fontset_table;
 extern int n_fonts, n_fontsets;
 
-extern void (*load_font_func)( /* struct frame *, FONT_INFO fontinfo, int lc */);
-extern char **(*font_list_func)( /* struct frame *f, char *name, int *count, int size */);
+extern void (*load_font_func)( /* FONT_INFO fontinfo, int lc */);
+extern char **(*font_list_func)( /* char *name, int *count, int size */);
