@@ -1,11 +1,11 @@
 /* Buffer insertion/deletion and gap motion for GNU Emacs.
-   Copyright (C) 1985, 1986, 1993, 1994, 1995 Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1993, 1994 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 1, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* 87.5.1   modified by K.Handa */
 /* 87.12.10 modified for Nemacs Ver.2.0 by K.Handa */
@@ -36,10 +35,8 @@ Boston, MA 02111-1307, USA.  */
 #include "window.h"
 #include "blockinput.h"
 
-#define min(x, y) ((x) < (y) ? (x) : (y))
-
+static void insert_1 ();
 static void insert_from_string_1 ();
-static void insert_from_buffer_1 ();
 static void gap_left ();
 static void gap_right ();
 static void adjust_markers ();
@@ -48,7 +45,6 @@ static void adjust_point ();
 /* Move gap to position `pos'.
    Note that this can quit!  */
 
-void
 move_gap (pos)
      int pos;
 {
@@ -141,7 +137,6 @@ gap_left (pos, newgap)
      or may be where a quit was detected.  */
   adjust_markers (pos + 1, GPT, GAP_SIZE);
   GPT = pos + 1;
-  if (GAP_SIZE > 0) *(GPT_ADDR) = 0; /* Put an anchor.  */
   QUIT;
 }
 
@@ -218,101 +213,71 @@ gap_right (pos)
 
   adjust_markers (GPT + GAP_SIZE, pos + 1 + GAP_SIZE, - GAP_SIZE);
   GPT = pos + 1;
-  if (GAP_SIZE > 0) *(GPT_ADDR) = 0; /* Put an anchor.  */
   QUIT;
 }
 
-/* Add AMOUNT to the position of every marker in the current buffer
-   whose current position is between FROM (exclusive) and TO (inclusive).
-
+/* Add `amount' to the position of every marker in the current buffer
+   whose current position is between `from' (exclusive) and `to' (inclusive).
    Also, any markers past the outside of that interval, in the direction
    of adjustment, are first moved back to the near end of the interval
-   and then adjusted by AMOUNT.
-
-   When the latter adjustment is done, if AMOUNT is negative,
-   we record the adjustment for undo.  (This case happens only for
-   deletion.)  */
+   and then adjusted by `amount'.  */
 
+/* 89.11.30 by K.Handa */
 static void
 adjust_markers (from, to, amount)
      register int from, to, amount;
+  {
+  adjust_markers2 (from, to, amount, MARKER_ALL_TYPE);
+}
+
+adjust_markers2 (from, to, amount, marker_type)
+     int from, to;
+     register int amount, marker_type;
 {
   Lisp_Object marker;
   register struct Lisp_Marker *m;
   register int mpos;
+  int range1, range2, newpos;
 
-  marker = BUF_MARKERS (current_buffer);
+  marker = current_buffer->markers;
+
+  if (amount > 0)
+    range1 = to, range2 = to + amount, newpos = range2;
+  else
+    range1 = from + amount, range2 = from + 1, newpos = range1;
 
   while (!NILP (marker))
     {
       m = XMARKER (marker);
-      mpos = m->bufpos;
-      if (amount > 0)
+      if (m->type & marker_type)
 	{
-	  if (mpos > to && mpos < to + amount)
-	    mpos = to + amount;
+	  mpos = m->bufpos;
+	  if (mpos > range1 && mpos < range2)
+	    mpos = newpos;
+	  if (mpos > from && mpos <= to)
+	    mpos += amount;
+	  m->bufpos = mpos;
 	}
-      else
-	{
-	  /* Here's the case where a marker is inside text being deleted.
-	     AMOUNT can be negative for gap motion, too,
-	     but then this range contains no markers.  */
-	  if (mpos > from + amount && mpos <= from)
-	    {
-	      record_marker_adjustment (marker, from + amount - mpos);
-	      mpos = from + amount;
-	    }
-	}
-      if (mpos > from && mpos <= to)
-	mpos += amount;
-      m->bufpos = mpos;
-      marker = m->chain;
-    }
-}
-
-/* Adjust markers whose insertion-type is t
-   for an insertion of AMOUNT characters at POS.  */
-
-static void
-adjust_markers_for_insert (pos, amount)
-     register int pos, amount;
-{
-  Lisp_Object marker;
-#ifdef MULE
-  extern Lisp_Object Vdisable_marker_relocation;
-
-  if (!NILP (Vdisable_marker_relocation))
-    return;
-#endif
-
-  marker = BUF_MARKERS (current_buffer);
-
-  while (!NILP (marker))
-    {
-      register struct Lisp_Marker *m = XMARKER (marker);
-      if (m->insertion_type && m->bufpos == pos)
-	m->bufpos += amount;
       marker = m->chain;
     }
 }
+/* end of patch */
 
 /* Add the specified amount to point.  This is used only when the value
    of point changes due to an insert or delete; it does not represent
    a conceptual change in point as a marker.  In particular, point is
    not crossing any interval boundaries, so there's no need to use the
    usual SET_PT macro.  In fact it would be incorrect to do so, because
-   either the old or the new value of point is out of sync with the
+   either the old or the new value of point is out of synch with the
    current set of intervals.  */
 static void
 adjust_point (amount)
-     int amount;
 {
-  BUF_PT (current_buffer) += amount;
+  current_buffer->text.pt += amount;
 }
 
 /* Make the gap INCREMENT characters longer.  */
 
-void
 make_gap (increment)
      int increment;
 {
@@ -324,28 +289,13 @@ make_gap (increment)
   /* If we have to get more space, get enough to last a while.  */
   increment += 2000;
 
-  /* Don't allow a buffer size that won't fit in an int
-     even if it will fit in a Lisp integer.
-     That won't work because so many places use `int'.  */
-     
-  if (Z - BEG + GAP_SIZE + increment
-      >= ((unsigned) 1 << (min (BITS_PER_INT, VALBITS) - 1)))
-    error ("Buffer exceeds maximum size");
-
   BLOCK_INPUT;
-  /* We allocate extra 1-byte at the tail and keep it always '\0' for
-     anchoring a search.  */
-  result = BUFFER_REALLOC (BEG_ADDR, (Z - BEG + GAP_SIZE + increment + 1));
+  result = BUFFER_REALLOC (BEG_ADDR, (Z - BEG + GAP_SIZE + increment));
+  UNBLOCK_INPUT;
 
   if (result == 0)
-    {
-      UNBLOCK_INPUT;
-      memory_full ();
-    }
-
-  /* We can't unblock until the new address is properly stored.  */
+    memory_full ();
   BEG_ADDR = result;
-  UNBLOCK_INPUT;
 
   /* Prevent quitting in move_gap.  */
   tem = Vinhibit_quit;
@@ -366,63 +316,79 @@ make_gap (increment)
   GAP_SIZE += old_gap_size;
   GPT = real_gap_loc;
 
-  /* Put an anchor.  */
-  *(Z_ADDR) = 0;
-
   Vinhibit_quit = tem;
 }
 
 /* Insert a string of specified length before point.
-   DO NOT use this for the contents of a Lisp string or a Lisp buffer!
-   prepare_to_modify_buffer could relocate the text.  */
+   DO NOT use this for the contents of a Lisp string!
+   prepare_to_modify_buffer could relocate the string.  */
 
-void
 insert (string, length)
      register unsigned char *string;
      register length;
 {
   if (length > 0)
     {
-      insert_1 (string, length, 0, 1);
+      register int opoint = PT;
+      insert_1 (string, length, 0);
+#ifdef MULE
+      adjust_markers2 (opoint - 1, opoint, length, MARKER_POINT_TYPE);
+      /* This insertion may result in putting point-type overlays
+	 in incorrect places in the list of overlays_before. */
+      fix_overlays_before (current_buffer, PT);
+#endif
       signal_after_change (PT-length, 0, length);
     }
 }
 
-void
 insert_and_inherit (string, length)
      register unsigned char *string;
      register length;
 {
   if (length > 0)
     {
-      insert_1 (string, length, 1, 1);
+      register int opoint = PT;
+      insert_1 (string, length, 1);
+#ifdef MULE
+      adjust_markers2 (opoint - 1, opoint, length, MARKER_POINT_TYPE);
+      /* This insertion may result in putting point-type overlays
+	 in incorrect places in the list of overlays_before. */
+      fix_overlays_before (current_buffer, PT);
+#endif
       signal_after_change (PT-length, 0, length);
     }
 }
 
-void
-insert_1 (string, length, inherit, prepare)
+static void
+insert_1 (string, length, inherit)
      register unsigned char *string;
-     register int length;
-     int inherit, prepare;
+     register length;
+     int inherit;
 {
   register Lisp_Object temp;
 
-  if (prepare)
-    prepare_to_modify_buffer (PT, PT);
+  /* Make sure point-max won't overflow after this insertion.  */
+  XSET (temp, Lisp_Int, length + Z);
+  if (length + Z != XINT (temp))
+    error ("maximum buffer size exceeded");
+
+  prepare_to_modify_buffer (PT, PT);
 
   if (PT != GPT)
     move_gap (PT);
-  if (GAP_SIZE < length)
+  if (GAP_SIZE < length + 1)
     make_gap (length - GAP_SIZE);
 
   record_insert (PT, length);
   MODIFF++;
 
   bcopy (string, GPT_ADDR, length);
+#ifdef MULE			/* 94.5.28 by K.Handa */
+  *(char *)(GPT_ADDR + length) = 0; /* Put an anchor */
+#endif  /* MULE */
 
 #ifdef USE_TEXT_PROPERTIES
-  if (BUF_INTERVALS (current_buffer) != 0)
+  if (current_buffer->intervals != 0)
     /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES.  */
     offset_intervals (current_buffer, PT, length);
 #endif
@@ -431,13 +397,10 @@ insert_1 (string, length, inherit, prepare)
   GPT += length;
   ZV += length;
   Z += length;
-  if (GAP_SIZE > 0) *(GPT_ADDR) = 0; /* Put an anchor.  */
-  adjust_overlays_for_insert (PT, length);
-  adjust_markers_for_insert (PT, length);
   adjust_point (length);
 
 #ifdef USE_TEXT_PROPERTIES
-  if (!inherit && BUF_INTERVALS (current_buffer) != 0)
+  if (!inherit && current_buffer->intervals != 0)
     Fset_text_properties (make_number (PT - length), make_number (PT),
 			  Qnil, Qnil);
 #endif
@@ -452,7 +415,6 @@ insert_1 (string, length, inherit, prepare)
    before we bcopy the stuff into the buffer, and relocate the string
    without insert noticing.  */
 
-void
 insert_from_string (string, pos, length, inherit)
      Lisp_Object string;
      register int pos, length;
@@ -460,7 +422,14 @@ insert_from_string (string, pos, length, inherit)
 {
   if (length > 0)
     {
+      register int opoint = PT;
       insert_from_string_1 (string, pos, length, inherit);
+#ifdef MULE
+      adjust_markers2 (opoint - 1, opoint, length, MARKER_POINT_TYPE);
+      /* This insertion may result in putting point-type overlays
+	 in incorrect places in the list of overlays_before. */
+      fix_overlays_before (current_buffer, PT);
+#endif
       signal_after_change (PT-length, 0, length);
     }
 }
@@ -475,7 +444,7 @@ insert_from_string_1 (string, pos, length, inherit)
   struct gcpro gcpro1;
 
   /* Make sure point-max won't overflow after this insertion.  */
-  XSETINT (temp, length + Z);
+  XSET (temp, Lisp_Int, length + Z);
   if (length + Z != XINT (temp))
     error ("maximum buffer size exceeded");
 
@@ -484,7 +453,7 @@ insert_from_string_1 (string, pos, length, inherit)
 
   if (PT != GPT)
     move_gap (PT);
-  if (GAP_SIZE < length)
+  if (GAP_SIZE < length + 1)
     make_gap (length - GAP_SIZE);
 
   record_insert (PT, length);
@@ -492,6 +461,9 @@ insert_from_string_1 (string, pos, length, inherit)
   UNGCPRO;
 
   bcopy (XSTRING (string)->data, GPT_ADDR, length);
+#ifdef MULE			/* 94.5.28 by K.Handa */
+  *(char *)(GPT_ADDR + length) = 0; /* Put an anchor */
+#endif  /* MULE */
 
   /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
   offset_intervals (current_buffer, PT, length);
@@ -500,9 +472,6 @@ insert_from_string_1 (string, pos, length, inherit)
   GPT += length;
   ZV += length;
   Z += length;
-  if (GAP_SIZE > 0) *(GPT_ADDR) = 0; /* Put an anchor.  */
-  adjust_overlays_for_insert (PT, length);
-  adjust_markers_for_insert (PT, length);
 
   /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
   graft_intervals_into_buffer (XSTRING (string)->intervals, PT, length,
@@ -511,83 +480,6 @@ insert_from_string_1 (string, pos, length, inherit)
   adjust_point (length);
 }
 
-/* Insert text from BUF, starting at POS and having length LENGTH, into the
-   current buffer.  If the text in BUF has properties, they are absorbed
-   into the current buffer.
-
-   It does not work to use `insert' for this, because a malloc could happen
-   and relocate BUF's text before the bcopy happens.  */
-
-void
-insert_from_buffer (buf, pos, length, inherit)
-     struct buffer *buf;
-     int pos, length;
-     int inherit;
-{
-  if (length > 0)
-    {
-      insert_from_buffer_1 (buf, pos, length, inherit);
-      signal_after_change (PT-length, 0, length);
-    }
-}
-
-static void
-insert_from_buffer_1 (buf, pos, length, inherit)
-     struct buffer *buf;
-     int pos, length;
-     int inherit;
-{
-  register Lisp_Object temp;
-  int chunk;
-
-  /* Make sure point-max won't overflow after this insertion.  */
-  XSETINT (temp, length + Z);
-  if (length + Z != XINT (temp))
-    error ("maximum buffer size exceeded");
-
-  prepare_to_modify_buffer (PT, PT);
-
-  if (PT != GPT)
-    move_gap (PT);
-  if (GAP_SIZE < length)
-    make_gap (length - GAP_SIZE);
-
-  record_insert (PT, length);
-  MODIFF++;
-
-  if (pos < BUF_GPT (buf))
-    {
-      chunk = BUF_GPT (buf) - pos;
-      if (chunk > length)
-	chunk = length;
-      bcopy (BUF_CHAR_ADDRESS (buf, pos), GPT_ADDR, chunk);
-    }
-  else
-    chunk = 0;
-  if (chunk < length)
-    bcopy (BUF_CHAR_ADDRESS (buf, pos + chunk),
-	   GPT_ADDR + chunk, length - chunk);
-
-#ifdef USE_TEXT_PROPERTIES
-  if (BUF_INTERVALS (current_buffer) != 0)
-    offset_intervals (current_buffer, PT, length);
-#endif
-
-  GAP_SIZE -= length;
-  GPT += length;
-  ZV += length;
-  Z += length;
-  if (GAP_SIZE > 0) *(GPT_ADDR) = 0; /* Put an anchor.  */
-  adjust_overlays_for_insert (PT, length);
-  adjust_markers_for_insert (PT, length);
-  adjust_point (length);
-
-  /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
-  graft_intervals_into_buffer (copy_intervals (BUF_INTERVALS (buf),
-					       pos, length),
-			       PT - length, length, current_buffer, inherit);
-}
-
 /* Insert the character C before point */
 
 void
@@ -611,7 +503,6 @@ insert_string (s)
    Don't use this function to insert part of a Lisp string,
    since gc could happen and relocate it.  */
 
-void
 insert_before_markers (string, length)
      unsigned char *string;
      register int length;
@@ -619,13 +510,12 @@ insert_before_markers (string, length)
   if (length > 0)
     {
       register int opoint = PT;
-      insert_1 (string, length, 0, 1);
+      insert_1 (string, length, 0);
       adjust_markers (opoint - 1, opoint, length);
       signal_after_change (PT-length, 0, length);
     }
 }
 
-void
 insert_before_markers_and_inherit (string, length)
      unsigned char *string;
      register int length;
@@ -633,7 +523,7 @@ insert_before_markers_and_inherit (string, length)
   if (length > 0)
     {
       register int opoint = PT;
-      insert_1 (string, length, 1, 1);
+      insert_1 (string, length, 1);
       adjust_markers (opoint - 1, opoint, length);
       signal_after_change (PT-length, 0, length);
     }
@@ -641,7 +531,6 @@ insert_before_markers_and_inherit (string, length)
 
 /* Insert part of a Lisp string, relocating markers after.  */
 
-void
 insert_from_string_before_markers (string, pos, length, inherit)
      Lisp_Object string;
      register int pos, length;
@@ -655,20 +544,60 @@ insert_from_string_before_markers (string, pos, length, inherit)
       signal_after_change (PT-length, 0, length);
     }
 }
+
+#ifdef MULE
+/* Like `insert' except that no markers (including point_type) are relocated.
+   Don't use this function to insert part of a Lisp string,
+   since gc could happen and relocate it.  */
+
+insert_after_markers (string, length)
+     unsigned char *string;
+     register int length;
+{
+  if (length > 0)
+    {
+      insert_1 (string, length, 0);
+      signal_after_change (PT-length, 0, length);
+    }
+}
+
+insert_after_markers_and_inherit (string, length)
+     unsigned char *string;
+     register int length;
+{
+  if (length > 0)
+    {
+      insert_1 (string, length, 1);
+      signal_after_change (PT-length, 0, length);
+    }
+}
+
+/* Insert part of a Lisp string.  */
+
+insert_from_string_after_markers (string, pos, length, inherit)
+     Lisp_Object string;
+     register int pos, length;
+     int inherit;
+{
+  if (length > 0)
+    {
+      insert_from_string_1 (string, pos, length, inherit);
+      signal_after_change (PT-length, 0, length);
+    }
+}
+#endif
 
 /* Delete characters in current buffer
    from FROM up to (but not including) TO.  */
 
-void
 del_range (from, to)
      register int from, to;
 {
-  del_range_1 (from, to, 1);
+  return del_range_1 (from, to, 1);
 }
 
 /* Like del_range; PREPARE says whether to call prepare_to_modify_buffer.  */
 
-void
 del_range_1 (from, to, prepare)
      register int from, to, prepare;
 {
@@ -692,12 +621,6 @@ del_range_1 (from, to, prepare)
   if (prepare)
     prepare_to_modify_buffer (from, to);
 
-  /* Relocate all markers pointing into the new, larger gap
-     to point at the end of the text before the gap.
-     This has to be done before recording the deletion,
-     so undo handles this after reinserting the text.  */
-  adjust_markers (to + GAP_SIZE, to + GAP_SIZE, - numdel - GAP_SIZE);
-
   record_delete (from, numdel);
   MODIFF++;
 
@@ -708,22 +631,20 @@ del_range_1 (from, to, prepare)
   /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
   offset_intervals (current_buffer, from, - numdel);
 
-  /* Adjust the overlay center as needed.  This must be done after
-     adjusting the markers that bound the overlays.  */
-  adjust_overlays_for_delete (from, numdel);
+  /* Relocate all markers pointing into the new, larger gap
+     to point at the end of the text before the gap.  */
+  adjust_markers (to + GAP_SIZE, to + GAP_SIZE, - numdel - GAP_SIZE);
 
   GAP_SIZE += numdel;
   ZV -= numdel;
   Z -= numdel;
   GPT = from;
-  *(GPT_ADDR) = 0;		/* Put an anchor.  */
 
   if (GPT - BEG < beg_unchanged)
     beg_unchanged = GPT - BEG;
   if (Z - GPT < end_unchanged)
     end_unchanged = Z - GPT;
 
-  evaporate_overlays (from);
   signal_after_change (from, numdel, 0);
 }
 
@@ -731,7 +652,6 @@ del_range_1 (from, to, prepare)
    to END.  This checks the read-only properties of the region, calls
    the necessary modification hooks, and warns the next redisplay that
    it should pay attention to that area.  */
-void
 modify_region (buffer, start, end)
      struct buffer *buffer;
      int start, end;
@@ -749,12 +669,10 @@ modify_region (buffer, start, end)
       || unchanged_modified == MODIFF)
     end_unchanged = Z - end;
 
-  if (MODIFF <= SAVE_MODIFF)
+  if (MODIFF <= current_buffer->save_modified)
     record_first_change ();
   MODIFF++;
 
-  buffer->point_before_scroll = Qnil;
-
   if (buffer != old_buffer)
     set_buffer_internal (old_buffer);
 }
@@ -764,7 +682,6 @@ modify_region (buffer, start, end)
    verify that the text to be modified is not read-only, and call
    any modification properties the text may have. */
 
-void
 prepare_to_modify_buffer (start, end)
      Lisp_Object start, end;
 {
@@ -772,19 +689,21 @@ prepare_to_modify_buffer (start, end)
     Fbarf_if_buffer_read_only ();
 
   /* Only defined if Emacs is compiled with USE_TEXT_PROPERTIES */
-  if (BUF_INTERVALS (current_buffer) != 0)
+  if (current_buffer->intervals != 0)
     verify_interval_modification (current_buffer, start, end);
 
+  if (!NILP (current_buffer->overlays_before)
+      || !NILP (current_buffer->overlays_after))
+    verify_overlay_modification (start, end);
+
 #ifdef CLASH_DETECTION
-  if (!NILP (current_buffer->file_truename)
-      /* Make binding buffer-file-name to nil effective.  */
-      && !NILP (current_buffer->filename)
-      && SAVE_MODIFF >= MODIFF)
-    lock_file (current_buffer->file_truename);
+  if (!NILP (current_buffer->filename)
+      && current_buffer->save_modified >= MODIFF)
+    lock_file (current_buffer->filename);
 #else
   /* At least warn if this file has changed on disk since it was visited.  */
   if (!NILP (current_buffer->filename)
-      && SAVE_MODIFF >= MODIFF
+      && current_buffer->save_modified >= MODIFF
       && NILP (Fverify_visited_file_modtime (Fcurrent_buffer ()))
       && !NILP (Ffile_exists_p (current_buffer->filename)))
     call1 (intern ("ask-user-about-supersession-threat"),
@@ -793,131 +712,163 @@ prepare_to_modify_buffer (start, end)
 
   signal_before_change (start, end);
 
-  if (current_buffer->newline_cache)
-    invalidate_region_cache (current_buffer,
-                             current_buffer->newline_cache,
-                             start - BEG, Z - end);
-  if (current_buffer->width_run_cache)
-    invalidate_region_cache (current_buffer,
-                             current_buffer->width_run_cache,
-                             start - BEG, Z - end);
-
   Vdeactivate_mark = Qt;
 }
 
+static Lisp_Object
+before_change_function_restore (value)
+     Lisp_Object value;
+{
+  Vbefore_change_function = value;
+}
+
+static Lisp_Object
+after_change_function_restore (value)
+     Lisp_Object value;
+{
+  Vafter_change_function = value;
+}
+
+static Lisp_Object
+before_change_functions_restore (value)
+     Lisp_Object value;
+{
+  Vbefore_change_functions = value;
+}
+
+static Lisp_Object
+after_change_functions_restore (value)
+     Lisp_Object value;
+{
+  Vafter_change_functions = value;
+}
+
 /* Signal a change to the buffer immediately before it happens.
    START and END are the bounds of the text to be changed,
    as Lisp objects.  */
 
-void
 signal_before_change (start, end)
      Lisp_Object start, end;
 {
   /* If buffer is unmodified, run a special hook for that case.  */
-  if (SAVE_MODIFF >= MODIFF
+  if (current_buffer->save_modified >= MODIFF
       && !NILP (Vfirst_change_hook)
       && !NILP (Vrun_hooks))
     call1 (Vrun_hooks, Qfirst_change_hook);
 
-  /* Run the before-change-function if any.
-     We don't bother "binding" this variable to nil
-     because it is obsolete anyway and new code should not use it.  */
+  /* Now in any case run the before-change-function if any.  */
   if (!NILP (Vbefore_change_function))
-    call2 (Vbefore_change_function, start, end);
-
-  /* Now run the before-change-functions if any.  */
-  if (!NILP (Vbefore_change_functions))
     {
-      Lisp_Object args[3];
-      Lisp_Object before_change_functions;
-      Lisp_Object after_change_functions;
-      struct gcpro gcpro1, gcpro2;
+      int count = specpdl_ptr - specpdl;
+      Lisp_Object function;
 
-      /* "Bind" before-change-functions and after-change-functions
-	 to nil--but in a way that errors don't know about.
-	 That way, if there's an error in them, they will stay nil.  */
-      before_change_functions = Vbefore_change_functions;
-      after_change_functions = Vafter_change_functions;
-      Vbefore_change_functions = Qnil;
+      function = Vbefore_change_function;
+
+      record_unwind_protect (after_change_function_restore,
+			     Vafter_change_function);
+      record_unwind_protect (before_change_function_restore,
+			     Vbefore_change_function);
+      record_unwind_protect (after_change_functions_restore,
+			     Vafter_change_functions);
+      record_unwind_protect (before_change_functions_restore,
+			     Vbefore_change_functions);
+      Vafter_change_function = Qnil;
+      Vbefore_change_function = Qnil;
       Vafter_change_functions = Qnil;
-      GCPRO2 (before_change_functions, after_change_functions);
+      Vbefore_change_functions = Qnil;
 
-      /* Actually run the hook functions.  */
-      args[0] = Qbefore_change_functions;
-      args[1] = start;
-      args[2] = end;
-      run_hook_list_with_args (before_change_functions, 3, args);
-
-      /* "Unbind" the variables we "bound" to nil.  */
-      Vbefore_change_functions = before_change_functions;
-      Vafter_change_functions = after_change_functions;
-      UNGCPRO;
+      call2 (function, start, end);
+      unbind_to (count, Qnil);
     }
 
-  if (!NILP (current_buffer->overlays_before)
-      || !NILP (current_buffer->overlays_after))
-    report_overlay_modification (start, end, 0, start, end, Qnil);
+  /* Now in any case run the before-change-function if any.  */
+  if (!NILP (Vbefore_change_functions))
+    {
+      int count = specpdl_ptr - specpdl;
+      Lisp_Object functions;
+
+      functions = Vbefore_change_functions;
+
+      record_unwind_protect (after_change_function_restore,
+			     Vafter_change_function);
+      record_unwind_protect (before_change_function_restore,
+			     Vbefore_change_function);
+      record_unwind_protect (after_change_functions_restore,
+			     Vafter_change_functions);
+      record_unwind_protect (before_change_functions_restore,
+			     Vbefore_change_functions);
+      Vafter_change_function = Qnil;
+      Vbefore_change_function = Qnil;
+      Vafter_change_functions = Qnil;
+      Vbefore_change_functions = Qnil;
+
+      while (CONSP (functions))
+	{
+	  call2 (XCONS (functions)->car, start, end);
+	  functions = XCONS (functions)->cdr;
+	}
+      unbind_to (count, Qnil);
+    }
 }
 
 /* Signal a change immediately after it happens.
    POS is the address of the start of the changed text.
    LENDEL is the number of characters of the text before the change.
    (Not the whole buffer; just the part that was changed.)
-   LENINS is the number of characters in that part of the text
-   after the change.  */
+   LENINS is the number of characters in the changed text.  */
 
-void
 signal_after_change (pos, lendel, lenins)
      int pos, lendel, lenins;
 {
-  /* Run the after-change-function if any.
-     We don't bother "binding" this variable to nil
-     because it is obsolete anyway and new code should not use it.  */
   if (!NILP (Vafter_change_function))
-    call3 (Vafter_change_function,
-	   make_number (pos), make_number (pos + lenins),
-	   make_number (lendel));
+    {
+      int count = specpdl_ptr - specpdl;
+      Lisp_Object function;
+      function = Vafter_change_function;
 
+      record_unwind_protect (after_change_function_restore,
+			     Vafter_change_function);
+      record_unwind_protect (before_change_function_restore,
+			     Vbefore_change_function);
+      record_unwind_protect (after_change_functions_restore,
+			     Vafter_change_functions);
+      record_unwind_protect (before_change_functions_restore,
+			     Vbefore_change_functions);
+      Vafter_change_function = Qnil;
+      Vbefore_change_function = Qnil;
+      Vafter_change_functions = Qnil;
+      Vbefore_change_functions = Qnil;
+
+      call3 (function, make_number (pos), make_number (pos + lenins),
+	     make_number (lendel));
+      unbind_to (count, Qnil);
+    }
   if (!NILP (Vafter_change_functions))
     {
-      Lisp_Object args[4];
-      Lisp_Object before_change_functions;
-      Lisp_Object after_change_functions;
-      struct gcpro gcpro1, gcpro2;
+      int count = specpdl_ptr - specpdl;
+      Lisp_Object functions;
+      functions = Vafter_change_functions;
 
-      /* "Bind" before-change-functions and after-change-functions
-	 to nil--but in a way that errors don't know about.
-	 That way, if there's an error in them, they will stay nil.  */
-      before_change_functions = Vbefore_change_functions;
-      after_change_functions = Vafter_change_functions;
-      Vbefore_change_functions = Qnil;
+      record_unwind_protect (after_change_function_restore,
+			     Vafter_change_function);
+      record_unwind_protect (before_change_function_restore,
+			     Vbefore_change_function);
+      record_unwind_protect (after_change_functions_restore,
+			     Vafter_change_functions);
+      record_unwind_protect (before_change_functions_restore,
+			     Vbefore_change_functions);
+      Vafter_change_function = Qnil;
+      Vbefore_change_function = Qnil;
       Vafter_change_functions = Qnil;
-      GCPRO2 (before_change_functions, after_change_functions);
+      Vbefore_change_functions = Qnil;
 
-      /* Actually run the hook functions.  */
-      args[0] = Qafter_change_functions;
-      XSETFASTINT (args[1], pos);
-      XSETFASTINT (args[2], pos + lenins);
-      XSETFASTINT (args[3], lendel);
-      run_hook_list_with_args (after_change_functions,
-			       4, args);
-
-      /* "Unbind" the variables we "bound" to nil.  */
-      Vbefore_change_functions = before_change_functions;
-      Vafter_change_functions = after_change_functions;
-      UNGCPRO;
+      while (CONSP (functions))
+	{
+	  call3 (XCONS (functions)->car,
+		 make_number (pos), make_number (pos + lenins),
+		 make_number (lendel));
+	  functions = XCONS (functions)->cdr;
+	}
+      unbind_to (count, Qnil);
     }
-
-  if (!NILP (current_buffer->overlays_before)
-      || !NILP (current_buffer->overlays_after))
-    report_overlay_modification (make_number (pos),
-				 make_number (pos + lenins),
-				 1,
-				 make_number (pos), make_number (pos + lenins),
-				 make_number (lendel));
-
-  /* After an insertion, call the text properties
-     insert-behind-hooks or insert-in-front-hooks.  */
-  if (lendel == 0)
-    report_interval_modification (pos, pos + lenins);
 }
