@@ -5,7 +5,7 @@ This file is part of GNU Emacs.
 
 GNU Emacs is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
+the Free Software Foundation; either version 1, or (at your option)
 any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
@@ -15,10 +15,9 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* Define PURESIZE, the number of bytes of pure Lisp code to leave space for.
+/* # bytes of pure Lisp code to leave space for.
 
    At one point, this was defined in config.h, meaning that changing
    PURESIZE would make Make recompile all of Emacs.  But only a few
@@ -28,38 +27,27 @@ Boston, MA 02111-1307, USA.  */
    whether we are running X windows, which tells us how much pure
    storage to allocate.  */
 
-/* First define a measure of the amount of data we have.  */
-
-/* A system configuration file may set this to request a certain extra
-   amount of storage.  This is a lot more update-robust that defining
-   BASE_PURESIZE or even PURESIZE directly.  */
-#ifndef SYSTEM_PURESIZE_EXTRA
-#define SYSTEM_PURESIZE_EXTRA 0
-#endif
-
-#ifndef SITELOAD_PURESIZE_EXTRA
-#define SITELOAD_PURESIZE_EXTRA 0
-#endif
-
 #ifndef BASE_PURESIZE
 #ifdef MULTI_FRAME
-#define BASE_PURESIZE (370000 + SYSTEM_PURESIZE_EXTRA + SITELOAD_PURESIZE_EXTRA)
+#define BASE_PURESIZE 295000
 #else
-#define BASE_PURESIZE (260000 + SYSTEM_PURESIZE_EXTRA + SITELOAD_PURESIZE_EXTRA)
+#define BASE_PURESIZE 275000
 #endif
 #endif
 
-/* Increase BASE_PURESIZE by a ratio depending on the machine's word size.  */
-#ifndef PURESIZE_RATIO
-#if VALBITS + GCTYPEBITS + 1 > 32
-#define PURESIZE_RATIO 8/5	/* Don't surround with `()'. */
+/* In 64 bit environment, PURESIZE should be multiplied by about 1.6. */
+/* LISP_OBJECT_BITS is defined in some -m files or config.h, 
+   which defaults to 32. */
+
+#ifndef PURESIZE_RATE
+#if LISP_OBJECT_BITS > 32
+#define PURESIZE_RATE 8/5	/* Don't surround with `()'. */
 #else
-#define PURESIZE_RATIO 1
+#define PURESIZE_RATE 1
 #endif
 #endif
 
-/* This is the actual size in bytes to allocate.  */
-#define TOTAL_PURESIZE ((BASE_PURESIZE + WNN_PURESIZE + SJ3_PURESIZE + EGG_PURESIZE + CANNA_PURESIZE) * PURESIZE_RATIO)
+#define TOTAL_PURESIZE ((BASE_PURESIZE + WNN_PURESIZE + SJ3_PURESIZE + EGG_PURESIZE + CANNA_PURESIZE) * PURESIZE_RATE)
 
 /* It will be safer for Mule to define PURESIZE here. */
 #ifdef MULE
@@ -72,38 +60,31 @@ Boston, MA 02111-1307, USA.  */
 #  define PURESIZE TOTAL_PURESIZE
 #endif /* PURESIZE */
 
-/* Signal an error if OBJ is pure.  */
-#define CHECK_IMPURE(obj) \
-  { if (PURE_P (obj))	  \
-      pure_write_error (); }
-
-/* Define PURE_P.  */
-
 #ifdef VIRT_ADDR_VARIES
+
 /* For machines like APOLLO where text and data can go anywhere
    in virtual memory.  */
-
-extern EMACS_INT pure[];
-
-#define PURE_P(obj) \
- ((PNTR_COMPARISON_TYPE) XPNTR (obj) < (PNTR_COMPARISON_TYPE) ((char *) pure + PURESIZE) \
-  && (PNTR_COMPARISON_TYPE) XPNTR (obj) >= (PNTR_COMPARISON_TYPE) pure)
+#define CHECK_IMPURE(obj) \
+  { extern Lisp_Object_Int pure[]; \
+    if ((PNTR_COMPARISON_TYPE) XPNTR (obj) < (PNTR_COMPARISON_TYPE) ((char *) pure + PURESIZE) \
+	&& (PNTR_COMPARISON_TYPE) XPNTR (obj) >= (PNTR_COMPARISON_TYPE) pure) \
+      pure_write_error (); }
 
 #else /* not VIRT_ADDR_VARIES */
 #ifdef PNTR_COMPARISON_TYPE
-/* When PNTR_COMPARISON_TYPE is not the default (unsigned int).  */
 
-extern char my_edata[];
-
-#define PURE_P(obj) \
-  ((PNTR_COMPARISON_TYPE) XPNTR (obj) < (PNTR_COMPARISON_TYPE) my_edata)
+/* when PNTR_COMPARISON_TYPE is not the default (unsigned int) */
+#define CHECK_IMPURE(obj) \
+  { extern Lisp_Object_Int my_edata; \
+    if ((PNTR_COMPARISON_TYPE) XPNTR (obj) < (PNTR_COMPARISON_TYPE) &my_edata) \
+      pure_write_error (); }
 
 #else /* not VIRT_ADDRESS_VARIES, not PNTR_COMPARISON_TYPE */
 
-extern char my_edata[];
-    
-#define PURE_P(obj) \
-  (XPNTR (obj) < (unsigned int) my_edata)
+#define CHECK_IMPURE(obj) \
+  { extern Lisp_Object_Int my_edata; \
+    if (XPNTR (obj) < (unsigned Lisp_Object_Int) &my_edata) \
+      pure_write_error (); }
 
 #endif /* PNTR_COMPARISON_TYPE */
 #endif /* VIRT_ADDRESS_VARIES */
