@@ -28,6 +28,9 @@
 
 #undef BSD_PGRPS
 
+/* The dumped Emacs records the timezone it was dumped in.  */
+#define LOCALTIME_CACHE
+
 /* System uses OXTABS instead of the expected TAB3.
    (Copied from netbsd.h.)  */
 #define TABDLY OXTABS
@@ -35,16 +38,13 @@
 
 #define SYSV_SYSTEM_DIR
 
-#define HAVE_TERMIOS
-#define NO_TERMIO
-
 #define	START_FILES	pre-crt0.o /usr/lib/crt0.o
 
-/* This silences a few compilation warnings.  */
-#ifdef emacs
+/* this silences a few compilation warnings */
+/*
 #undef BSD
-#include <sys/param.h> /* To get BSD defined consistently.  */
-#endif
+#define BSD 199103
+*/
 
 #define WAITTYPE int
 /* get this since it won't be included if WAITTYPE is defined */
@@ -55,5 +55,3 @@
 #ifndef WCOREDUMP
 #define WCOREDUMP(w) ((w) & 0200)
 #endif
-
-#define GETPGRP_NO_ARG 1
