@@ -15,21 +15,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Emacs 19 face widget ported by Fred Pierresteguy */
 
-/* This file has been censored by the Communications Decency Act.
-   That law was passed under the guise of a ban on pornography, but
-   it bans far more than that.  This file did not contain pornography,
-   but it was censored nonetheless.
-
-   For information on US government censorship of the Internet, and
-   what you can do to bring back freedom of the press, see the web
-   site http://www.vtw.org/
-   */
-
 #include <config.h>
 #include <stdio.h>
 #include "lisp.h"
@@ -37,7 +26,7 @@ Boston, MA 02111-1307, USA.  */
 #include "xterm.h"
 
 #include "dispextern.h"
-#include "blockinput.h"
+
 
 #include <X11/StringDefs.h>
 #include <X11/IntrinsicP.h>
@@ -46,7 +35,6 @@ Boston, MA 02111-1307, USA.  */
 #include <X11/ObjectP.h>
 #include <X11/Shell.h>
 #include <X11/ShellP.h>
-#include "../lwlib/lwlib.h"
 
 #define max(a, b) ((a) > (b) ? (a) : (b))
 
@@ -179,8 +167,8 @@ get_default_char_pixel_size (ew, pixel_width, pixel_height)
      int* pixel_height;
 {
   struct frame* f = ew->emacs_frame.frame;
-  *pixel_width = FONT_WIDTH (f->output_data.x->font);
-  *pixel_height = f->output_data.x->line_height;
+  *pixel_width = FONT_WIDTH (f->display.x->font);
+  *pixel_height = f->display.x->line_height;
 }
 
 static void
@@ -192,8 +180,8 @@ pixel_to_char_size (ew, pixel_width, pixel_height, char_width, char_height)
      int* char_height;
 {
   struct frame* f = ew->emacs_frame.frame;
-  *char_width = PIXEL_TO_CHAR_WIDTH (f, (int) pixel_width);
-  *char_height = PIXEL_TO_CHAR_HEIGHT (f, (int) pixel_height);
+  *char_width = PIXEL_TO_CHAR_WIDTH (f, pixel_width);
+  *char_height = PIXEL_TO_CHAR_HEIGHT (f, pixel_height);
 }
 
 static void
@@ -316,12 +304,12 @@ set_frame_size (ew)
   int flags = 0;
   
   Widget wmshell = get_wm_shell ((Widget) ew);
-  /* Each Emacs shell is now independent and top-level.  */
-  Widget app_shell = wmshell;
+  Widget app_shell = XtParent ((Widget) wmshell);
   
   if (! XtIsSubclass (wmshell, shellWidgetClass)) abort ();
+  if (! XtIsSubclass (app_shell, shellWidgetClass)) abort ();
 
-  /* We don't need this for the moment. The geometry is computed in 
+  /* We don't need this for the momment. The geometry is computed in 
      xfns.c.  */
 #if 0
   /* If the EmacsFrame doesn't have a geometry but the shell does,
@@ -433,13 +421,11 @@ set_frame_size (ew)
     char shell_position [32];
 
     /* Take into account the size of the scrollbar */
-    frame->output_data.x->vertical_scroll_bar_extra
-      = (!FRAME_HAS_VERTICAL_SCROLL_BARS (frame)
-	 ? 0
-	 : FRAME_SCROLL_BAR_PIXEL_WIDTH (frame) > 0
-	 ? FRAME_SCROLL_BAR_PIXEL_WIDTH (frame)
-	 : (FRAME_SCROLL_BAR_COLS (frame)
-	    * FONT_WIDTH (frame->output_data.x->font)));
+    frame->display.x->vertical_scroll_bar_extra
+      = (FRAME_HAS_VERTICAL_SCROLL_BARS (frame)
+	 ? VERTICAL_SCROLL_BAR_PIXEL_WIDTH (frame)
+	 : 0);
+
 
     change_frame_size (frame, h, w, 1, 0);
     char_to_pixel_size (ew, w, h, &pixel_width, &pixel_height);
@@ -487,9 +473,6 @@ set_frame_size (ew)
   }
 }
 
-/* Nonzero tells update_wm_hints not to do anything
-   (the caller should call update_wm_hints explicitly later.)  */
-int update_hints_inhibit;
 
 static void
 update_wm_hints (ew)
@@ -506,12 +489,7 @@ update_wm_hints (ew)
   int base_height;
   int min_rows = 0, min_cols = 0;
 
-  if (update_hints_inhibit)
-    return;
-
-#if 0
   check_frame_size (ew->emacs_frame.frame, &min_rows, &min_cols);
-#endif
 
   pixel_to_char_size (ew, ew->core.width, ew->core.height,
 		      &char_width, &char_height);
@@ -543,17 +521,14 @@ static void
 create_frame_gcs (ew)
      EmacsFrame ew;
 {
-  struct frame *s = ew->emacs_frame.frame;
+  struct frame* s = ew->emacs_frame.frame;
 
-  s->output_data.x->normal_gc
-    = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
-		 (unsigned long)0, (XGCValues *)0);
-  s->output_data.x->reverse_gc
-    = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
-		 (unsigned long)0, (XGCValues *)0);
-  s->output_data.x->cursor_gc
-    = XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
-		 (unsigned long)0, (XGCValues *)0);
+  s->display.x->normal_gc =
+    XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)), 0, 0);
+  s->display.x->reverse_gc =
+    XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)), 0, 0);
+  s->display.x->cursor_gc =
+    XCreateGC (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)), 0, 0);
 }
 
 static char setup_frame_cursor_bits[] =
@@ -581,20 +556,18 @@ setup_frame_gcs (ew)
      effectively cache all of the GC settings we need to use.
    */
 
-  blank_stipple
-    = XCreateBitmapFromData (XtDisplay (ew),
-			     RootWindowOfScreen (XtScreen (ew)),
-			     setup_frame_cursor_bits, 2, 2);
+  blank_stipple = 
+    XCreateBitmapFromData (XtDisplay (ew), RootWindowOfScreen (XtScreen (ew)),
+			   setup_frame_cursor_bits, 2, 2);
 
   /* use fg = 0, bg = 1 below, but it's irrelevant since this pixmap should
      never actually get used as a background tile!
    */
-  blank_tile
-    = XCreatePixmapFromBitmapData (XtDisplay(ew),
-				   RootWindowOfScreen (XtScreen (ew)),
-				   setup_frame_cursor_bits, 2, 2,
-				   (unsigned long)0, (unsigned long)1,
-				   ew->core.depth);
+  blank_tile =
+    XCreatePixmapFromBitmapData (XtDisplay(ew),
+				 RootWindowOfScreen (XtScreen (ew)),
+				 setup_frame_cursor_bits, 2, 2, 0, 1,
+				 ew->core.depth);
 
   /* Normal video */
   gc_values.font = ew->emacs_frame.font->fid;
@@ -603,7 +576,7 @@ setup_frame_gcs (ew)
   gc_values.graphics_exposures = False;
   gc_values.stipple = blank_stipple;
   gc_values.tile = blank_tile;
-  XChangeGC (XtDisplay (ew), s->output_data.x->normal_gc,
+  XChangeGC (XtDisplay (ew), s->display.x->normal_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -615,7 +588,7 @@ setup_frame_gcs (ew)
   gc_values.graphics_exposures = False;
   gc_values.stipple = blank_stipple;
   gc_values.tile = blank_tile;
-  XChangeGC (XtDisplay (ew), s->output_data.x->reverse_gc,
+  XChangeGC (XtDisplay (ew), s->display.x->reverse_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -626,11 +599,11 @@ setup_frame_gcs (ew)
   gc_values.background = ew->emacs_frame.cursor_color;
   gc_values.graphics_exposures = False;
   gc_values.tile = blank_tile;
-  gc_values.stipple
-    = XCreateBitmapFromData (XtDisplay (ew),
-			     RootWindowOfScreen (XtScreen (ew)),
-			     setup_frame_cursor_bits, 16, 16);
-  XChangeGC (XtDisplay (ew), s->output_data.x->cursor_gc,
+  gc_values.stipple =
+    XCreateBitmapFromData (XtDisplay (ew),
+			   RootWindowOfScreen (XtScreen (ew)),
+			   setup_frame_cursor_bits, 16, 16);
+  XChangeGC (XtDisplay (ew), s->display.x->cursor_gc,
 	     (GCFont | GCForeground | GCBackground | GCGraphicsExposures
 	      | GCStipple | GCTile),
 	     &gc_values);
@@ -640,8 +613,8 @@ static void
 update_various_frame_slots (ew)
      EmacsFrame ew;
 {
-  struct x_output *x = ew->emacs_frame.frame->output_data.x;
-  x->pixel_height = ew->core.height + x->menubar_height;
+  struct x_display* x = ew->emacs_frame.frame->display.x;
+  x->pixel_height = ew->core.height;
   x->pixel_width = ew->core.width;
   x->internal_border_width = ew->emacs_frame.internal_border_width;
 
@@ -651,8 +624,8 @@ static void
 update_from_various_frame_slots (ew)
      EmacsFrame ew;
 {
-  struct x_output *x = ew->emacs_frame.frame->output_data.x;
-  ew->core.height = x->pixel_height - x->menubar_height;
+  struct x_display* x = ew->emacs_frame.frame->display.x;
+  ew->core.height = x->pixel_height;
   ew->core.width = x->pixel_width;
   ew->core.background_pixel = x->background_pixel;
   ew->emacs_frame.internal_border_width = x->internal_border_width;
@@ -712,7 +685,7 @@ EmacsFrameInitialize (request, new, dum1, dum2)
   }
 
 /* Update the font field in frame */
-  ew->emacs_frame.frame->output_data.x->font = ew->emacs_frame.font;
+  ew->emacs_frame.frame->display.x->font = ew->emacs_frame.font;
 #endif
 
   update_from_various_frame_slots (ew);
@@ -731,8 +704,13 @@ EmacsFrameRealize (widget, mask, attrs)
 {
   EmacsFrame ew = (EmacsFrame)widget;
 
-  attrs->event_mask = (STANDARD_EVENT_SET | PropertyChangeMask
-		       | SubstructureNotifyMask | SubstructureRedirectMask);
+  attrs->event_mask = (KeyPressMask | ExposureMask | ButtonPressMask |
+		       ButtonReleaseMask | StructureNotifyMask |
+		       FocusChangeMask | PointerMotionHintMask |
+		       PointerMotionMask | LeaveWindowMask | EnterWindowMask |
+		       VisibilityChangeMask | PropertyChangeMask |
+		       StructureNotifyMask | SubstructureNotifyMask |
+		       SubstructureRedirectMask);
   *mask |= CWEventMask;
   XtCreateWindow (widget, InputOutput, (Visual *)CopyFromParent, *mask,
 		  attrs);
@@ -749,20 +727,18 @@ EmacsFrameDestroy (widget)
   struct frame* s = ew->emacs_frame.frame;
 
   if (! s) abort ();
-  if (! s->output_data.x) abort ();
-  if (! s->output_data.x->normal_gc) abort ();
+  if (! s->display.x) abort ();
+  if (! s->display.x->normal_gc) abort ();
 
   /* this would be called from Fdelete_frame() but it needs to free some
      stuff after the widget has been finalized but before the widget has
      been freed. */
   free_frame_faces (s);
 
-  BLOCK_INPUT;
   /* need to be careful that the face-freeing code doesn't free these too */
-  XFreeGC (XtDisplay (widget), s->output_data.x->normal_gc);
-  XFreeGC (XtDisplay (widget), s->output_data.x->reverse_gc);
-  XFreeGC (XtDisplay (widget), s->output_data.x->cursor_gc);
-  UNBLOCK_INPUT;
+  XFreeGC (XtDisplay (widget), s->display.x->normal_gc);
+  XFreeGC (XtDisplay (widget), s->display.x->reverse_gc);
+  XFreeGC (XtDisplay (widget), s->display.x->cursor_gc);
 }
 
 void
@@ -775,7 +751,7 @@ EmacsFrameResize (widget)
   int rows;
 
   pixel_to_char_size (ew, ew->core.width, ew->core.height, &columns, &rows);
-  change_frame_size (f, rows, columns, 0, 1);
+  change_frame_size (f, rows, columns, 1, 0);
   update_wm_hints (ew); 
   update_various_frame_slots (ew);
 }
@@ -845,7 +821,7 @@ EmacsFrameSetValues (cur_widget, req_widget, new_widget, dum1, dum2)
      to wm_shell's iconic slot have no effect after it has been realized,
      and calling XIconifyWindow doesn't work either (even thought the window
      has been created.)  Perhaps there is some property we could smash
-     directly, but I'm sick of this for now.
+     directly, but I'm sick of this for now.  Xt is a steaming pile of shit!
    */
   if (cur->emacs_frame.iconic != new->emacs_frame.iconic)
     {
@@ -905,30 +881,21 @@ EmacsFrameSetCharSize (widget, columns, rows)
   if (columns < 3) columns = 3;  /* no way buddy */
 
   check_frame_size (f, &rows, &columns);
-  f->output_data.x->vertical_scroll_bar_extra
-    = (!FRAME_HAS_VERTICAL_SCROLL_BARS (f)
-       ? 0
-       : FRAME_SCROLL_BAR_PIXEL_WIDTH (f) > 0
-       ? FRAME_SCROLL_BAR_PIXEL_WIDTH (f)
-       : (FRAME_SCROLL_BAR_COLS (f) * FONT_WIDTH (f->output_data.x->font)));
-
+  f->display.x->vertical_scroll_bar_extra
+    = (FRAME_HAS_VERTICAL_SCROLL_BARS (f)
+       ? VERTICAL_SCROLL_BAR_PIXEL_WIDTH (f)
+       : 0);
   char_to_pixel_size (ew, columns, rows, &pixel_width, &pixel_height);
 
-  /* Manually change the height and width of all our widgets,
-     adjusting each widget by the same increments.  */
+  /* Recompute the entire geometry management.  */
   if (ew->core.width != pixel_width || ew->core.height != pixel_height)
     {
       int hdelta = pixel_height - ew->core.height;
-      int wdelta = pixel_width - ew->core.width;
-      int column_widget_height = f->output_data.x->column_widget->core.height;
-      int column_widget_width = f->output_data.x->column_widget->core.width;
-      int outer_widget_height = f->output_data.x->widget->core.height;
-      int outer_widget_width = f->output_data.x->widget->core.width;
-      int old_left = f->output_data.x->widget->core.x;
-      int old_top = f->output_data.x->widget->core.y;
+      int column_widget_height = f->display.x->column_widget->core.height;
+      int old_left = f->display.x->widget->core.x;
+      int old_top = f->display.x->widget->core.y;
 
-      lw_refigure_widget (f->output_data.x->column_widget, False);
-      update_hints_inhibit = 1;
+      XawPanedSetRefigureMode (f->display.x->column_widget, False);
 
       ac = 0;
       XtSetArg (al[ac], XtNheight, pixel_height); ac++;
@@ -937,24 +904,14 @@ EmacsFrameSetCharSize (widget, columns, rows)
  
       ac = 0;
       XtSetArg (al[ac], XtNheight, column_widget_height + hdelta); ac++;
-      XtSetArg (al[ac], XtNwidth, column_widget_width + wdelta); ac++;
-      XtSetValues (f->output_data.x->column_widget, al, ac);
+      XtSetArg (al[ac], XtNwidth, pixel_width); ac++;
+      XtSetValues (f->display.x->column_widget, al, ac);
 
-      ac = 0;
-      XtSetArg (al[ac], XtNheight, outer_widget_height + hdelta); ac++;
-      XtSetArg (al[ac], XtNwidth, outer_widget_width + wdelta); ac++;
-      XtSetValues (f->output_data.x->widget, al, ac);
-
-      lw_refigure_widget (f->output_data.x->column_widget, True);
-
-      update_hints_inhibit = 0;
-      update_wm_hints (ew);
-
-      do_pending_window_change ();
+      XawPanedSetRefigureMode (f->display.x->column_widget, True);
 
       /* These seem to get clobbered.  I don't know why. - rms.  */
-      f->output_data.x->widget->core.x = old_left;
-      f->output_data.x->widget->core.y = old_top;
+      f->display.x->widget->core.x = old_left;
+      f->display.x->widget->core.y = old_top;
     }
 
   /* We've set {FRAME,PIXEL}_{WIDTH,HEIGHT} to the values we hope to
