@@ -1,22 +1,19 @@
 /* Deal with the X Resource Manager.
    Copyright (C) 1990, 1993, 1994 Free Software Foundation.
 
-This file is part of GNU Emacs.
-
-GNU Emacs is free software; you can redistribute it and/or modify
+This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
-GNU Emacs is distributed in the hope that it will be useful,
+This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with this program; see the file COPYING.  If not, write to
+the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by jla, 4/90 */
 
@@ -37,6 +34,10 @@ Boston, MA 02111-1307, USA.  */
 
 #endif /* 1 */
 
+/* This should be included before the X include files; otherwise, we get
+   warnings about redefining NULL under BSD 4.3.  */
+#include <sys/param.h>
+
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #if 0
@@ -52,6 +53,10 @@ Boston, MA 02111-1307, USA.  */
 #endif
 #include <sys/stat.h>
 
+#ifndef MAXPATHLEN
+#define MAXPATHLEN	256
+#endif
+
 #if !defined(S_ISDIR) && defined(S_IFDIR)
 #define S_ISDIR(m) (((m) & S_IFMT) == S_IFDIR)
 #endif
@@ -84,6 +89,16 @@ extern char *get_system_name ();
 #define free xfree
 #endif
 
+#ifdef OSF1
+#if __STDC__
+extern void *malloc();
+extern void *realloc();
+#else
+extern char *malloc();
+extern char *realloc();
+#endif
+#endif
+
 char *x_get_string_resource ();
 static int file_p ();
 
